from numba.pycc import CC
from numpy import zeros

cc = CC('CNum_inner_compiled')


@cc.export('CNum_inner',
           '(int64, int64[:,::1],float64[:,:,::1],float64[::1],int64,float64[:,:,::1],float64[:,::1],float64[:,:,::1],boolean[::1],float64[:,:,::1])')
def CNum_inner(NYrs, DaysMonth, Temp, CN, NRur, melt_pest, newcn, amc5, grow_factor, water):
    result = zeros((NYrs, 12, 31, NRur))  # TODO: should we just generalize to NLU?
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                if Temp[Y][i][j] > 0 and water[Y][i][j] > 0.01:  # forgot this
                    for l in range(NRur):
                        if CN[l] > 0:
                            if melt_pest[Y][i][j] <= 0:
                                if grow_factor[i] == True:
                                    # growing season
                                    if amc5[Y][i][j] >= 5.33:  # forgot "get value from yesterday"
                                        result[Y][i][j][l] = newcn[2][l]
                                    elif amc5[Y][i][j] < 3.56:
                                        result[Y][i][j][l] = newcn[0][l] + (
                                                CN[l] - newcn[0][l]) * amc5[Y][i][j] / 3.56
                                    else:
                                        result[Y][i][j][l] = CN[l] + (newcn[2][l] - CN[l]) * (
                                                amc5[Y][i][j] - 3.56) / 1.77
                                else:
                                    # dormant season
                                    if amc5[Y][i][j] >= 2.79:
                                        result[Y][i][j][l] = newcn[2][l]
                                    elif amc5[Y][i][j] < 1.27:
                                        result[Y][i][j][l] = newcn[0][l] + (
                                                CN[l] - newcn[0][l]) * amc5[Y][i][j] / 1.27
                                    else:
                                        result[Y][i][j][l] = CN[l] + (newcn[2][l] - CN[l]) * (
                                                amc5[Y][i][j] - 1.27) / 1.52
                            else:
                                result[Y][i][j][l] = newcn[2][l]
    return result



from numpy import sum

from .GRLoadN import GRLoadN
from .GRLoadN import GRLoadN_f
from gwlfe.Input.Animals.GrazingAnimal import GrazingAnimal
from gwlfe.Memoization import memoize
from gwlfe.enums import YesOrNo


def InitGrN(GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN):
    result = 0
    gr_load_n = GRLoadN(GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN)
    grazing_animal = GrazingAnimal(GrazingAnimal_0)
    for a in range(9):
        if grazing_animal[a] is YesOrNo.NO:
            pass
        elif grazing_animal[a] is YesOrNo.YES:
            result += gr_load_n[a]
    return result


@memoize
def InitGrN_f(GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN):
    return sum(GRLoadN_f(GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN))



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.AFOS.GrazingAnimals.Loads import GrazingN


class TestGrazingN(VariableUnitTest):
    def test_GrazingN(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            GrazingN.GrazingN(z.PctGrazing, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN)[None, :],
            GrazingN.GrazingN_f(z.PctGrazing, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN),
            decimal=7)



import csv
import os
import unittest
from io import StringIO
import json

import numpy as np

from gwlfe import Parser
from gwlfe import gwlfe



class TestGMSWriter(unittest.TestCase):

    @classmethod
    def setUpClass(self):
        self.input_file = open(os.path.abspath(os.path.join(__file__, '../', "mapshed_data.json.txt")), "r")
        self.output_file = open(os.path.abspath(os.path.join(__file__, '../', "mapshed_data.gms")), 'r')

    def test_gms_writer(self):
        """
        Test that GmsWriter is able to replicate the sample GMS created
        from MapShed.
        """
        mapshed_data = json.load(self.input_file)
        pre_z = Parser.DataModel(mapshed_data)
        output = StringIO()
        writer = Parser.GmsWriter(output)
        writer.write(pre_z)
        output.seek(0)

        ground_truth = csv.reader(self.output_file, delimiter=",")
        output_parsed = csv.reader(output, delimiter=",")
        error = False
        for i, row in enumerate(zip(ground_truth, output_parsed)):
            for j, column in enumerate(zip(row[0], row[1])):
                ground_truth_val = column[0]
                output_val = column[1]
                try:
                    try:
                        np.testing.assert_allclose(float(ground_truth_val), float(output_val), rtol=1e-07)
                    except ValueError:  # catch all non float values
                        self.assertEqual(ground_truth_val, output_val)
                except AssertionError as e:
                    print("Error on line %i, column %i" % (i + 1, j))
                    print(e)
                    error = True
        if (error == True):
            raise AssertionError("Not all variables within margin of error")



from gwlfe.Memoization import memoize
from gwlfe.enums import YesOrNo


@memoize
def GrazingAnimal(GrazingAnimal_0):
    return GrazingAnimal_0


@memoize
def GrazingAnimal_f(GrazingAnimal_0):
    return GrazingAnimal_0 == YesOrNo.YES



# -*- coding: utf-8 -*-
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

"""
Imported from AnnualMeans.bas
"""

import logging
from .Input.WaterBudget import Precipitation
from .Input.WaterBudget.AvEvapoTrans import AvEvapoTrans_f
from .MultiUse_Fxns.PtSrcFlow import AvPtSrcFlow_f
from .MultiUse_Fxns.Erosion.AvStreamBankEros import AvStreamBankEros_f
from .Input.LandUse.Ag.AvTileDrain import AvTileDrain_f
from .Input.WaterBudget.AvWithdrawal import AvWithdrawal_f
from .Input.WaterBudget.AvGroundWater import AvGroundWater_f
from .MultiUse_Fxns.Runoff.AvRunoff import AvRunoff_f
from .MultiUse_Fxns.Erosion.AvErosion import AvErosion_f
from .MultiUse_Fxns.Erosion.AvSedYield import AvSedYield_f
from .MultiUse_Fxns.Runoff.LuRunoff import LuRunoff_f
from .Output.Loading.LuTotPhos import LuTotPhos_f
from .Output.Loading.LuTotNitr import LuTotNitr_f

log = logging.getLogger(__name__)


def CalculateAnnualMeanLoads(z, Y):
    # UPDATE SEPTIC SYSTEM AVERAGES
    z.AvSeptNitr += z.SepticNitr[Y] / z.NYrs
    z.AvSeptPhos += z.SepticPhos[Y] / z.NYrs

    z.CalendarYr = z.WxYrBeg + (Y - 1)

    # CALCULATE ANNUAL MEANS FOR STREAM BANK AND TILE DRAINAGE VALUES
    z.AvPtSrcFlow = AvPtSrcFlow_f(z.PointFlow)
    for i in range(12):
        z.AvStreamBankP[i] += z.StreamBankP[Y][i] / z.NYrs
        z.AvTileDrainN[i] += z.TileDrainN[Y][i] / z.NYrs
        z.AvTileDrainP[i] += z.TileDrainP[Y][i] / z.NYrs
        z.AvTileDrainSed[i] += z.TileDrainSed[Y][i] / z.NYrs

    # COMPUTE ANNUAL MEANS
    z.AvPrecipitation = Precipitation.AvPrecipitation_f(Precipitation.Precipitation_f(z.Prec))
    for i in range(12):
        z.AvDisNitr[i] += z.DisNitr[Y][i] / z.NYrs
        z.AvTotNitr[i] += z.TotNitr[Y][i] / z.NYrs
        z.AvDisPhos[i] += z.DisPhos[Y][i] / z.NYrs
        z.AvTotPhos[i] += z.TotPhos[Y][i] / z.NYrs
        z.AvGroundNitr[i] += z.GroundNitr[Y][i] / z.NYrs
        z.AvGroundPhos[i] += z.GroundPhos[Y][i] / z.NYrs
        z.AvAnimalP[i] += z.AnimalP[Y][i] / z.NYrs

        z.AvGRLostBarnP[i] += z.GRLostBarnP[Y][i] / z.NYrs
        z.AvGRLostBarnFC[i] += z.GRLostBarnFC[Y][i] / z.NYrs

        z.AvNGLostBarnP[i] += z.NGLostBarnP[Y][i] / z.NYrs
        z.AvNGLostBarnFC[i] += z.NGLostBarnFC[Y][i] / z.NYrs

        z.AvNGLostManP[i] += z.NGLostManP[Y][i] / z.NYrs

        # Average pathogen totals
        z.AvAnimalFC[i] += z.AnimalFC[Y][i] / z.NYrs
        z.AvWWOrgs[i] += z.WWOrgs[Y][i] / z.NYrs
        z.AvSSOrgs[i] += z.SSOrgs[Y][i] / z.NYrs
        z.AvUrbOrgs[i] += z.UrbOrgs[Y][i] / z.NYrs
        z.AvWildOrgs[i] += z.WildOrgs[Y][i] / z.NYrs
        z.AvTotalOrgs[i] += z.TotalOrgs[Y][i] / z.NYrs

    # Average loads for each landuse
    for l in range(z.NRur):
        z.AvLuRunoff[l] += \
            LuRunoff_f(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.NRur, z.NUrb, z.CNI_0, z.CNP_0,
                       z.AntMoist_0, z.Grow_0, z.Imper, z.ISRR, z.ISRA, z.CN)[Y][l] / z.NYrs
        z.AvLuErosion[l] += \
            LuRunoff_f(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.NRur, z.NUrb, z.CNI_0, z.CNP_0,
                       z.AntMoist_0, z.Grow_0, z.Imper, z.ISRR, z.ISRA, z.CN)[Y][l] / z.NYrs
        z.AvLuSedYield[l] += z.LuSedYield[Y][l] / z.NYrs
        z.AvLuDisNitr[l] += z.LuDisNitr[Y][l] / z.NYrs
        z.AvLuTotNitr[l] += \
        LuTotNitr_f(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.AntMoist_0, z.NRur, z.NUrb, z.CN, z.Grow_0,
                    z.Area, z.NitrConc, z.ManNitr, z.ManuredAreas, z.FirstManureMonth, z.LastManureMonth,
                    z.FirstManureMonth2, z.LastManureMonth2, z.SedDelivRatio_0, z.KF, z.LS, z.C, z.P, z.SedNitr,
                    z.CNP_0, z.Imper, z.ISRR, z.ISRA,
                    z.Qretention, z.PctAreaInfil, z.LoadRateImp, z.LoadRatePerv, z.Storm, z.UrbBMPRed, z.FilterWidth,
                    z.PctStrmBuf, z.Acoef,
                    z.CNI_0, z.Nqual)[Y][l] / z.NYrs
        z.AvLuDisPhos[l] += z.LuDisPhos[Y][l] / z.NYrs
        z.AvLuTotPhos[l] += \
        LuTotPhos_f(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.AntMoist_0, z.NRur, z.NUrb, z.CN,
                    z.Grow_0, z.Area, z.PhosConc, z.ManPhos, z.ManuredAreas, z.FirstManureMonth,
                    z.LastManureMonth, z.FirstManureMonth2, z.LastManureMonth2, z.SedDelivRatio_0, z.KF,
                    z.LS, z.C, z.P, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.Qretention, z.PctAreaInfil, z.Nqual,
                    z.LoadRateImp, z.LoadRatePerv, z.Storm, z.UrbBMPRed, z.FilterWidth, z.PctStrmBuf,
                    z.Acoef, z.SedPhos, z.CNI_0)[Y][l] / z.NYrs

    for l in range(z.NRur, z.NLU):
        z.AvLuRunoff[l] += \
            LuRunoff_f(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.NRur, z.NUrb, z.CNI_0, z.CNP_0,
                       z.AntMoist_0, z.Grow_0, z.Imper, z.ISRR, z.ISRA, z.CN)[Y][l] / z.NYrs
        z.AvLuTotNitr[l] += \
        LuTotNitr_f(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.AntMoist_0, z.NRur, z.NUrb, z.CN, z.Grow_0,
                    z.Area, z.NitrConc, z.ManNitr, z.ManuredAreas, z.FirstManureMonth, z.LastManureMonth,
                    z.FirstManureMonth2, z.LastManureMonth2, z.SedDelivRatio_0, z.KF, z.LS, z.C, z.P, z.SedNitr,
                    z.CNP_0, z.Imper, z.ISRR, z.ISRA,
                    z.Qretention, z.PctAreaInfil, z.LoadRateImp, z.LoadRatePerv, z.Storm, z.UrbBMPRed, z.FilterWidth,
                    z.PctStrmBuf, z.Acoef,
                    z.CNI_0, z.Nqual)[Y][l] / z.NYrs
        z.AvLuTotPhos[l] += \
        LuTotPhos_f(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.AntMoist_0, z.NRur, z.NUrb, z.CN,
                    z.Grow_0, z.Area, z.PhosConc, z.ManPhos, z.ManuredAreas, z.FirstManureMonth,
                    z.LastManureMonth, z.FirstManureMonth2, z.LastManureMonth2, z.SedDelivRatio_0, z.KF,
                    z.LS, z.C, z.P, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.Qretention, z.PctAreaInfil, z.Nqual,
                    z.LoadRateImp, z.LoadRatePerv, z.Storm, z.UrbBMPRed, z.FilterWidth, z.PctStrmBuf,
                    z.Acoef, z.SedPhos, z.CNI_0)[Y][l] / z.NYrs
        z.AvLuDisNitr[l] += z.LuDisNitr[Y][l] / z.NYrs
        z.AvLuDisPhos[l] += z.LuDisPhos[Y][l] / z.NYrs
        z.AvLuSedYield[l] += z.LuSedYield[Y][l] / z.NYrs

    z.AvStreamBankErosSum = sum(
        AvStreamBankEros_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                           z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV,
                           z.PcntET,
                           z.DayHrs, z.MaxWaterCap, z.SatStor_0, z.RecessionCoef, z.SeepCoef, z.Qretention,
                           z.PctAreaInfil, z.n25b, z.Landuse, z.TileDrainDensity, z.PointFlow, z.StreamWithdrawal,
                           z.GroundWithdrawal, z.NumAnimals, z.AvgAnimalWt, z.StreamFlowVolAdj, z.SedAFactor_0, z.AvKF,
                           z.AvSlope, z.SedAAdjust, z.StreamLength, z.n42b, z.n46c, z.n85d, z.AgLength, z.n42, z.n45,
                           z.n85, z.UrbBankStab))
    z.AvStreamBankPSum = sum(z.AvStreamBankP)
    z.AvPtSrcFlowSum = sum(z.AvPtSrcFlow)
    z.AvTileDrainSum = sum(AvTileDrain_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area,
                                         z.CNI_0, z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper,
                                         z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV, z.PcntET, z.DayHrs, z.MaxWaterCap,
                                         z.SatStor_0, z.RecessionCoef, z.SeepCoef,
                                         z.Landuse, z.TileDrainDensity))
    z.AvWithdrawalSum = sum(AvWithdrawal_f(z.NYrs, z.StreamWithdrawal, z.GroundWithdrawal))
    z.AvTileDrainNSum = sum(z.AvTileDrainN)
    z.AvTileDrainPSum = sum(z.AvTileDrainP)
    z.AvTileDrainSedSum = sum(z.AvTileDrainSed)
    z.AvEvapoTransSum = sum(
        AvEvapoTrans_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0, z.AntMoist_0,
                       z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV, z.PcntET, z.DayHrs,
                       z.MaxWaterCap))
    z.AvGroundWaterSum = sum(
        AvGroundWater_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                        z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV, z.PcntET,
                        z.DayHrs, z.MaxWaterCap,
                        z.SatStor_0, z.RecessionCoef, z.SeepCoef, z.Landuse, z.TileDrainDensity))
    z.AvRunoffSum = sum(AvRunoff_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                                   z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.Qretention,
                                   z.PctAreaInfil,
                                   z.n25b, z.CN, z.Landuse, z.TileDrainDensity))
    z.AvErosionSum = sum(
        AvErosion_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0, z.AntMoist_0,
                    z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV, z.PcntET, z.DayHrs,
                    z.MaxWaterCap,
                    z.SatStor_0, z.RecessionCoef, z.SeepCoef, z.Qretention, z.PctAreaInfil, z.n25b, z.Landuse,
                    z.TileDrainDensity, z.PointFlow,
                    z.StreamWithdrawal, z.GroundWithdrawal, z.NumAnimals, z.AvgAnimalWt, z.StreamFlowVolAdj,
                    z.SedAFactor_0,
                    z.AvKF, z.AvSlope, z.SedAAdjust, z.StreamLength, z.n42b, z.n46c, z.n85d, z.AgLength, z.n42, z.n45,
                    z.n85, z.UrbBankStab,
                    z.SedDelivRatio_0, z.Acoef, z.KF, z.LS, z.C, z.P))
    z.AvSedYieldSum = sum(
        AvSedYield_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                     z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV,
                     z.PcntET, z.DayHrs, z.MaxWaterCap, z.SatStor_0, z.RecessionCoef, z.SeepCoef,
                     z.Qretention, z.PctAreaInfil, z.n25b, z.Landuse, z.TileDrainDensity, z.PointFlow,
                     z.StreamWithdrawal, z.GroundWithdrawal, z.NumAnimals, z.AvgAnimalWt,
                     z.StreamFlowVolAdj, z.SedAFactor_0, z.AvKF, z.AvSlope, z.SedAAdjust, z.StreamLength,
                     z.n42b, z.n46c, z.n85d, z.AgLength, z.n42, z.n45, z.n85, z.UrbBankStab, z.Acoef, z.KF,
                     z.LS, z.C, z.P, z.SedDelivRatio_0))
    z.AvDisNitrSum = sum(z.AvDisNitr)
    z.AvTotNitrSum = sum(z.AvTotNitr)
    z.AvDisPhosSum = sum(z.AvDisPhos)
    z.AvTotPhosSum = sum(z.AvTotPhos)
    z.AvGroundNitrSum = sum(z.AvGroundNitr)
    z.AvGroundPhosSum = sum(z.AvGroundPhos)
    z.AvAnimalPSum = sum(z.AvAnimalP)
    z.AvGRLostBarnPSum = sum(z.AvGRLostBarnP)
    z.AvGRLostBarnFCSum = sum(z.AvGRLostBarnFC)
    z.AvNGLostBarnPSum = sum(z.AvNGLostBarnP)
    z.AvNGLostBarnFCSum = sum(z.AvNGLostBarnFC)
    z.AvNGLostManPSum = sum(z.AvNGLostManP)
    z.AvAnimalFCSum = sum(z.AvAnimalFC)
    z.AvWWOrgsSum = sum(z.AvWWOrgs)
    z.AvSSOrgsSum = sum(z.AvSSOrgs)
    z.AvUrbOrgsSum = sum(z.AvUrbOrgs)
    z.AvWildOrgsSum = sum(z.AvWildOrgs)
    z.AvTotalOrgsSum = sum(z.AvTotalOrgs)
    z.AvLuRunoffSum = sum(z.AvLuRunoff)
    z.AvLuErosionSum = sum(z.AvLuErosion)
    z.AvLuSedYieldSum = sum(z.AvLuSedYield)
    z.AvLuDisNitrSum = sum(z.AvLuDisNitr)
    z.AvLuTotNitrSum = sum(z.AvLuTotNitr)
    z.AvLuDisPhosSum = sum(z.AvLuDisPhos)
    z.AvLuTotPhosSum = sum(z.AvLuTotPhos)



from numpy import repeat
from numpy import tile
from numpy import where
from numpy import zeros

from gwlfe.Input.LandUse.NLU import NLU
from gwlfe.Input.WaterBudget.Water import Water, Water_f
from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Runoff.CNP import CNP, CNP_f
from gwlfe.MultiUse_Fxns.Runoff.CNumPerv import CNumPerv, CNumPerv_f


@memoize
def CNumPervReten(NYrs, DaysMonth, Temp, Prec, InitSnow_0, AntMoist_0, NRur, NUrb, CNP_0, Grow_0):
    cnp = CNP(NRur, NUrb, CNP_0)
    c_num_perv = CNumPerv(NYrs, DaysMonth, Temp, NRur, NUrb, CNP_0, InitSnow_0, Prec, Grow_0, AntMoist_0)
    nlu = NLU(NRur, NUrb)
    water = Water(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    result = zeros((NYrs, 12, 31, nlu))
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                if Temp[Y][i][j] > 0 and water[Y][i][j] > 0.01:  # missing
                    if water[Y][i][j] < 0.05:  # missing
                        pass
                    else:
                        for l in range(NRur, nlu):
                            if cnp[1][l] > 0:
                                result[Y][i][j][l] = 2540 / c_num_perv[Y][i][j][l] - 25.4
                                if result[Y][i][j][l] < 0:
                                    result[Y][i][j][l] = 0
    return result


def CNumPervReten_f(NYrs, DaysMonth, Temp, Prec, InitSnow_0, AntMoist_0, NRur, NUrb, CNP_0, Grow_0):
    nlu = NLU(NRur, NUrb)
    result = zeros((NYrs, 12, 31, nlu))
    c_num_perv = CNumPerv_f(NYrs, DaysMonth, Temp, NRur, NUrb, CNP_0, InitSnow_0, Prec, Grow_0, AntMoist_0)
    cnp = CNP_f(NRur, NUrb, CNP_0)
    cnp_1 = tile(cnp[1][None, None, None, :], (NYrs, 12, 31, 1))
    water = repeat(Water_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec)[:, :, :, None], nlu, axis=3)
    Temp = repeat(Temp[:, :, :, None], nlu, axis=3)
    result[where((Temp > 0) & (water >= 0.05) & (cnp_1 > 0))] = 2540 / c_num_perv[
        where((Temp > 0) & (water >= 0.05) & (cnp_1 > 0))] - 25.4
    result[where(result < 0)] = 0
    return result



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns.Erosion import AvErosion


class TestAvErosion(VariableUnitTest):

    def test_elementwise_AvErosion(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            np.load(self.basepath + "/AvErosion.npy"),
            AvErosion.AvErosion(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                                z.AntMoist_0,
                                z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV, z.PcntET,
                                z.DayHrs, z.MaxWaterCap,
                                z.SatStor_0, z.RecessionCoef, z.SeepCoef, z.Qretention, z.PctAreaInfil, z.n25b,
                                z.Landuse, z.TileDrainDensity, z.PointFlow,
                                z.StreamWithdrawal, z.GroundWithdrawal, z.NumAnimals, z.AvgAnimalWt, z.StreamFlowVolAdj,
                                z.SedAFactor_0,
                                z.AvKF, z.AvSlope, z.SedAAdjust, z.StreamLength, z.n42b, z.n46c, z.n85d, z.AgLength,
                                z.n42, z.n45, z.n85, z.UrbBankStab,
                                z.SedDelivRatio_0, z.Acoef, z.KF, z.LS, z.C, z.P), decimal=7)

    def test_AvErosion(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            AvErosion.AvErosion_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                                  z.AntMoist_0,
                                  z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV, z.PcntET,
                                  z.DayHrs, z.MaxWaterCap,
                                  z.SatStor_0, z.RecessionCoef, z.SeepCoef, z.Qretention, z.PctAreaInfil, z.n25b,
                                  z.Landuse, z.TileDrainDensity, z.PointFlow,
                                  z.StreamWithdrawal, z.GroundWithdrawal, z.NumAnimals, z.AvgAnimalWt,
                                  z.StreamFlowVolAdj, z.SedAFactor_0,
                                  z.AvKF, z.AvSlope, z.SedAAdjust, z.StreamLength, z.n42b, z.n46c, z.n85d, z.AgLength,
                                  z.n42, z.n45, z.n85, z.UrbBankStab,
                                  z.SedDelivRatio_0, z.Acoef, z.KF, z.LS, z.C, z.P),
            AvErosion.AvErosion(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                                z.AntMoist_0,
                                z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV, z.PcntET,
                                z.DayHrs, z.MaxWaterCap,
                                z.SatStor_0, z.RecessionCoef, z.SeepCoef, z.Qretention, z.PctAreaInfil, z.n25b,
                                z.Landuse, z.TileDrainDensity, z.PointFlow,
                                z.StreamWithdrawal, z.GroundWithdrawal, z.NumAnimals, z.AvgAnimalWt, z.StreamFlowVolAdj,
                                z.SedAFactor_0,
                                z.AvKF, z.AvSlope, z.SedAAdjust, z.StreamLength, z.n42b, z.n46c, z.n85d, z.AgLength,
                                z.n42, z.n45, z.n85, z.UrbBankStab,
                                z.SedDelivRatio_0, z.Acoef, z.KF, z.LS, z.C, z.P), decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns.Runoff import NewCN


class TestNewCN(VariableUnitTest):
    def test_NewCN(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            NewCN.NewCN_f(z.NRur, z.NUrb, z.CN),
            NewCN.NewCN(z.NRur, z.NUrb, z.CN), decimal=7)



from numpy import zeros

from gwlfe.Input.WaterBudget.Melt_1 import Melt_1, Melt_1_f
from gwlfe.Input.WaterBudget.Rain import Rain, Rain_f
from gwlfe.Memoization import memoize


@memoize
def Water(NYrs, DaysMonth, InitSnow_0, Temp, Prec):
    result = zeros((NYrs, 12, 31))
    melt_1 = Melt_1(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    rain = Rain(NYrs, DaysMonth, Temp, Prec)
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                result[Y][i][j] = rain[Y][i][j] + melt_1[Y][i][j]
    return result


@memoize
def Water_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec):
    melt_1 = Melt_1_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    rain = Rain_f(Temp, Prec)
    return rain + melt_1



from numpy import sum
from numpy import zeros

from gwlfe.Memoization import memoize
from gwlfe.Output.Loading.StreamBankN_1 import StreamBankN_1
from gwlfe.Output.Loading.StreamBankN_1 import StreamBankN_1_f


@memoize
def StreamBankNSum(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area,
                   CNI_0, AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, CN,
                   UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                   RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse,
                   TileDrainDensity, PointFlow, StreamWithdrawal, GroundWithdrawal,
                   NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF,
                   AvSlope, SedAAdjust, StreamLength, n42b, AgLength,
                   UrbBankStab, SedNitr, BankNFrac, n69c, n45, n69, n46c, n42):
    result = zeros((NYrs,))
    streambank_n_1 = StreamBankN_1(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area,
                                   CNI_0, AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, CN,
                                   UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                                   RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse,
                                   TileDrainDensity, PointFlow, StreamWithdrawal, GroundWithdrawal,
                                   NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF,
                                   AvSlope, SedAAdjust, StreamLength, n42b, AgLength,
                                   UrbBankStab, SedNitr, BankNFrac, n69c, n45, n69, n46c, n42)
    for Y in range(NYrs):
        for i in range(12):
            result[Y] += streambank_n_1[Y][i]

    return result


@memoize
def StreamBankNSum_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area,
                     CNI_0, AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, CN,
                     UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                     RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse,
                     TileDrainDensity, PointFlow, StreamWithdrawal, GroundWithdrawal,
                     NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF,
                     AvSlope, SedAAdjust, StreamLength, n42b, AgLength,
                     UrbBankStab, SedNitr, BankNFrac, n69c, n45, n69, n46c, n42):
    return sum(StreamBankN_1_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area,
                               CNI_0, AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, CN,
                               UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                               RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse,
                               TileDrainDensity, PointFlow, StreamWithdrawal, GroundWithdrawal,
                               NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF,
                               AvSlope, SedAAdjust, StreamLength, n42b, AgLength,
                               UrbBankStab, SedNitr, BankNFrac, n69c, n45, n69, n46c, n42), axis=1)



from numpy import where
from numpy import zeros

from gwlfe.Memoization import memoize
# from Timer import time_function
from gwlfe.MultiUse_Fxns.Discharge.StreamFlow import StreamFlow
from gwlfe.MultiUse_Fxns.Discharge.StreamFlow import StreamFlow_f


@memoize
def StreamFlowLE(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                 ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef, SeepCoef
                 , Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow, StreamWithdrawal,
                 GroundWithdrawal):
    result = zeros((NYrs, 12))
    streamflow = StreamFlow(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0,
                            Imper,
                            ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef,
                            SeepCoef
                            , Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow, StreamWithdrawal,
                            GroundWithdrawal)
    for Y in range(NYrs):
        for i in range(12):
            result[Y][i] = streamflow[Y][i]
            if result[Y][i] < 0:
                result[Y][i] = 0
    return result


def StreamFlowLE_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                   ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef, SeepCoef,
                   Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow, StreamWithdrawal,
                   GroundWithdrawal):
    streamflow = StreamFlow_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0,
                              CNP_0,
                              Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                              RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity,
                              PointFlow, StreamWithdrawal, GroundWithdrawal)
    return where(streamflow > 0, streamflow, 0)



from numpy import zeros

from gwlfe.Memoization import memoize
from gwlfe.Output.Loading.SedYield import SedYield
from gwlfe.Output.Loading.SedYield import SedYield_f
from gwlfe.Output.Loading.StreamBankEros_1 import StreamBankEros_1
from gwlfe.Output.Loading.StreamBankEros_1 import StreamBankEros_1_f


@memoize
def SedYield_1(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
               Grow_0, CNP_0, Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
               RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow,
               StreamWithdrawal, GroundWithdrawal, NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0,
               AvKF, AvSlope, SedAAdjust, StreamLength, n42b, n46c, n85d, AgLength, n42, n45, n85, UrbBankStab,
               Acoef, KF, LS, C, P, SedDelivRatio_0):
    result = zeros((NYrs, 12))
    sedyield = SedYield(NYrs, DaysMonth, Temp, InitSnow_0, Prec, Acoef, NRur, KF, LS, C, P, Area, NUrb, CNI_0,
                        AntMoist_0, Grow_0,
                        ISRR, ISRA, Qretention, PctAreaInfil, n25b, CN, CNP_0, Imper, SedDelivRatio_0)
    streambankeros_f = StreamBankEros_1(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
                                        Grow_0, CNP_0, Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs,
                                        MaxWaterCap, SatStor_0,
                                        RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse,
                                        TileDrainDensity, PointFlow,
                                        StreamWithdrawal, GroundWithdrawal, NumAnimals, AvgAnimalWt, StreamFlowVolAdj,
                                        SedAFactor_0,
                                        AvKF, AvSlope, SedAAdjust, StreamLength, n42b, n46c, n85d, AgLength, n42, n45,
                                        n85, UrbBankStab)
    for Y in range(NYrs):
        for i in range(12):
            result[Y][i] = sedyield[Y][i] + streambankeros_f[Y][i] / 1000
    return result


@memoize
def SedYield_1_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
                 Grow_0, CNP_0, Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                 RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow,
                 StreamWithdrawal, GroundWithdrawal, NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0,
                 AvKF, AvSlope, SedAAdjust, StreamLength, n42b, n46c, n85d, AgLength, n42, n45, n85, UrbBankStab,
                 Acoef, KF, LS, C, P, SedDelivRatio_0):
    return SedYield_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, Acoef, NRur, KF, LS, C, P, Area, NUrb, CNI_0, AntMoist_0,
                      Grow_0, ISRR, ISRA, Qretention, PctAreaInfil, n25b, CN, CNP_0, Imper,
                      SedDelivRatio_0) + StreamBankEros_1_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area,
                                                            CNI_0, AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, CN,
                                                            UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                                                            RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b,
                                                            Landuse, TileDrainDensity, PointFlow, StreamWithdrawal,
                                                            GroundWithdrawal, NumAnimals, AvgAnimalWt, StreamFlowVolAdj,
                                                            SedAFactor_0, AvKF, AvSlope, SedAAdjust, StreamLength, n42b,
                                                            n46c, n85d, AgLength, n42, n45, n85, UrbBankStab) / 1000



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.AFOS.GrazingAnimals.Losses import GRLostManN


class TestGRLostManN(VariableUnitTest):

    def test_GRLostManN(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            GRLostManN.GRLostManN(z.NYrs, z.GRPctManApp, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN,
                                  z.GRAppNRate, z.Prec, z.DaysMonth, z.GRPctSoilIncRate),
            GRLostManN.GRLostManN_f(z.NYrs, z.GRPctManApp, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt,
                                    z.AnimalDailyN, z.GRAppNRate, z.Prec, z.DaysMonth, z.GRPctSoilIncRate), decimal=7)



# -*- coding: utf-8 -*-
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from .test_output import TestOutput


# from mock import patch
# from gwlfe.Memoization import memoize_with_args
# patch('gwlfe.Memoization.memoize', memoize_with_args).start()


class input_4_TestOutput(TestOutput):
    """
    Tests model generated output versus known
    static output.
    """
    __test__ = True
    @classmethod
    def setUpClass(self):
        super(input_4_TestOutput, self).setUpClass('input_4.gms', 'input_4_output.json')



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns.Discharge import AdjUrbanQTotal_1


class TestAdjUrbanQTotal_1(VariableUnitTest):

    def test_AdjUrbanQTotal_1(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            AdjUrbanQTotal_1.AdjUrbanQTotal_1_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb,
                                                z.Area,
                                                z.CNI_0, z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA,
                                                z.Qretention, z.PctAreaInfil),
            AdjUrbanQTotal_1.AdjUrbanQTotal_1(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area,
                                              z.CNI_0, z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA,
                                              z.Qretention, z.PctAreaInfil),
            decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.BMPs.AgAnimal import NFENCING


class TestNFENCING(VariableUnitTest):
    def test_NFENCING(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            NFENCING.NFENCING_f(z.PctStreams, z.PctGrazing, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt,
                                z.AnimalDailyN, z.n42, z.n45, z.n69),
            NFENCING.NFENCING(z.PctStreams, z.PctGrazing, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt,
                              z.AnimalDailyN, z.n42, z.n45, z.n69), decimal=7)



# from Timer import time_function
from numpy import zeros

from gwlfe.Input.LandUse.AreaTotal import AreaTotal
from gwlfe.Input.LandUse.AreaTotal import AreaTotal_f
from gwlfe.Input.LandUse.Urb.UrbAreaTotal import UrbAreaTotal
from gwlfe.Input.LandUse.Urb.UrbAreaTotal import UrbAreaTotal_f
from gwlfe.Input.WaterBudget.Water import Water
from gwlfe.Input.WaterBudget.Water import Water_f
from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Discharge.UrbanQTotal import UrbanQTotal
from gwlfe.MultiUse_Fxns.Discharge.UrbanQTotal import UrbanQTotal_f

try:
    from .AdjUrbanQTotal_inner_compiled import AdjUrbanQTotal_inner
except ImportError:
    print("Unable to import compiled AdjUrbanQTotal_inner, using slower version")
    from gwlfe.MultiUse_Fxns.Discharge.AdjUrbanQTotal_inner import AdjUrbanQTotal_inner


# @time_function
@memoize
# @time_function
def AdjUrbanQTotal(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                   ISRR, ISRA, Qretention, PctAreaInfil):
    result = zeros((NYrs, 12, 31))
    adj_urban_q_total = 0  # used because this is a buffered variable
    water = Water(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    urban_q_total = UrbanQTotal(NYrs, DaysMonth, NRur, NUrb, Temp, InitSnow_0, Prec, Area, CNI_0, AntMoist_0, Grow_0,
                                CNP_0, Imper, ISRR, ISRA)
    urb_area_total = UrbAreaTotal(NRur, NUrb, Area)
    area_total = AreaTotal(NRur, NUrb, Area)
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                if Temp[Y][i][j] > 0 and water[Y][i][j] > 0.01:
                    if water[Y][i][j] < 0.05:
                        ## z.adj_urban_q_total = get_value_for_yesterday(z.adj_urban_q_total_1,0,Y,i,j,z.NYrs,z.DaysMonth)
                        adj_urban_q_total *= urb_area_total / area_total
                        # pass
                    else:
                        adj_urban_q_total = urban_q_total[Y][i][j]
                        if Qretention > 0:
                            if urban_q_total[Y][i][j] > 0:
                                if urban_q_total[Y][i][j] <= Qretention * PctAreaInfil:
                                    adj_urban_q_total = 0
                                else:
                                    adj_urban_q_total = urban_q_total[Y][i][j] - Qretention * PctAreaInfil
                    # if urb_area_total > 0:
                    #     adj_urban_q_total = adj_urban_q_total * urb_area_total / area_total
                    # else:
                    #     adj_urban_q_total = 0
                else:
                    pass
                result[Y][i][j] = adj_urban_q_total
    return result


@memoize
def AdjUrbanQTotal_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                     ISRR, ISRA, Qretention, PctAreaInfil):
    water = Water_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    urban_q_total = UrbanQTotal_f(NYrs, DaysMonth, NRur, NUrb, Temp, InitSnow_0, Prec, Area, CNI_0, AntMoist_0, Grow_0,
                                  CNP_0, Imper, ISRR, ISRA)
    urb_area_total = UrbAreaTotal_f(NRur, NUrb, Area)
    area_total = AreaTotal_f(Area)
    return AdjUrbanQTotal_inner(NYrs, DaysMonth, Temp, Qretention, PctAreaInfil, water, urban_q_total, urb_area_total,
                                area_total)

# @jit
# def AdjUrbanQTotal_f_inner(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
#                    ISRR, ISRA, Qretention, PctAreaInfil,water,urban_q_total,urb_area_total,area_total):
#     result = np.zeros((NYrs, 12, 31))
#     adj_urban_q_total = 0
#     urban_q_total = DAC.ymd_to_daily(urban_q_total, DaysMonth)
#     Temp = DAC.ymd_to_daily(Temp, DaysMonth)
#     water = DAC.ymd_to_daily(water, DaysMonth)
#     for j in range(len(urban_q_total)):
#         if Temp[j] > 0 and water[j] > 0.01:
#             if water[j] <0.05:
#                 pass
#             else:
#                 adj_urban_q_total = urban_q_total[j]
#                 if Qretention > 0 and urban_q_total[j] > 0:
#                     if urban_q_total[j] <= Qretention * PctAreaInfil:
#                         adj_urban_q_total = 0
#                     else:
#                         adj_urban_q_total = urban_q_total[j] - Qretention * PctAreaInfil
#             if urb_area_total > 0:
#                 adj_urban_q_total = adj_urban_q_total * urb_area_total / area_total
#             else:
#                 adj_urban_q_total = 0
#         else:
#             pass
#         result[j] = adj_urban_q_total
#     return DAC.daily_to_ymd(result,NYrs, DaysMonth)



from numpy import sum
from numpy import zeros

from gwlfe.Input.LandUse.NLU import NLU
from gwlfe.Input.WaterBudget.Water import Water
from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Erosion.RurEros import RurEros
from gwlfe.MultiUse_Fxns.Erosion.RurEros import RurEros_f


@memoize
def ErosWashoff(NYrs, DaysMonth, InitSnow_0, Temp, Prec, NRur, NUrb, Acoef, KF, LS, C, P, Area):
    result = zeros((NYrs, 16, 12))
    nlu = NLU(NRur, NUrb)
    water = Water(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    rureros = RurEros(NYrs, DaysMonth, Temp, InitSnow_0, Prec, Acoef, NRur, KF, LS, C, P, Area)
    for Y in range(NYrs):
        for i in range(12):
            for l in range(nlu):
                result[Y, l, i] = 0
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                if Temp[Y][i][j] > 0 and water[Y][i][j] > 0.01:
                    for l in range(NRur):
                        result[Y][l][i] = result[Y][l][i] + rureros[Y][i][j][l]
                else:
                    pass
    return result


@memoize
def ErosWashoff_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec, NRur, Acoef, KF, LS, C, P, Area):
    return sum(RurEros_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, Acoef, NRur, KF, LS, C, P, Area), axis=2)



# import unittest
# from unittest import skip
# from mock import patch
# import numpy as np
# from gwlfe import Parser
# from gwlfe import AvStreamBankN
#
#
# class TestAvStreamBankN(VariableUnitTest):
#     def setUp(self):
#         input_file = open('unittests/input_4.gms', 'r')
#         self.z = Parser.GmsReader(input_file).read()
#
#
#     @skip("not ready")
#     def test_AvStreamBankN(self):
#         z = self.z
#         np.testing.assert_array_almost_equal(
#             AvStreamBankN.AvStreamBankN_f(),
#             AvStreamBankN.AvStreamBankN(), decimal=7)



from numpy import sum
from numpy import zeros

from gwlfe.Input.WaterBudget.GroundWatLE_1 import GroundWatLE_1
from gwlfe.Input.WaterBudget.GroundWatLE_1 import GroundWatLE_1_f
from gwlfe.Memoization import memoize


def AvGroundWater(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0,
                  AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap,
                  SatStor_0, RecessionCoef, SeepCoef, Landuse, TileDrainDensity):
    result = zeros(12)
    ground_wat_le = GroundWatLE_1(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0,
                                  CNP_0, Imper,
                                  ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                                  RecessionCoef, SeepCoef,
                                  Landuse, TileDrainDensity)
    for Y in range(NYrs):
        for i in range(12):
            result[i] += ground_wat_le[Y][i] / NYrs
    return result


@memoize
def AvGroundWater_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0,
                    AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap,
                    SatStor_0, RecessionCoef, SeepCoef, Landuse, TileDrainDensity):
    return sum(GroundWatLE_1_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0,
                               AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs,
                               MaxWaterCap, SatStor_0, RecessionCoef, SeepCoef, Landuse, TileDrainDensity),
               axis=0) / NYrs



from numpy import sum
from numpy import zeros

from gwlfe.Input.WaterBudget.Withdrawal import Withdrawal
from gwlfe.Input.WaterBudget.Withdrawal import Withdrawal_f
from gwlfe.Memoization import memoize


def AvWithdrawal(NYrs, StreamWithdrawal, GroundWithdrawal):
    result = zeros(12)
    withdrawal = Withdrawal(NYrs, StreamWithdrawal, GroundWithdrawal)
    for Y in range(NYrs):
        for i in range(12):
            result[i] += withdrawal[Y][i] / NYrs
    return result


@memoize
def AvWithdrawal_f(NYrs, StreamWithdrawal, GroundWithdrawal):
    return sum(Withdrawal_f(NYrs, StreamWithdrawal, GroundWithdrawal), axis=0) / NYrs



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.Input.WaterBudget import Water


class TestWater(VariableUnitTest):

    def test_Water_ground_truth(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            np.load(self.basepath + "/Water.npy"),
            Water.Water(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec), decimal=7)

    def test_Water(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            Water.Water_f(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec),
            Water.Water(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec), decimal=7)



from unittest import skip

import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.BMPs.AgAnimal import NAWSL_2


class TestNAWSL_2(VariableUnitTest):
    @skip('Not Ready Yet.')
    def test_NAWSL_2(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            NAWSL_2.NAWSL_2_f(),
            NAWSL_2.NAWSL_2(), decimal=7)



from unittest import skip

import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.AFOS.nonGrazingAnimals.Losses import NGLostBarnNSum


class TestNGLostBarnNSum(VariableUnitTest):
    @skip('Not Ready Yet.')
    def test_NGLostBarnNSum(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            NGLostBarnNSum.NGLostBarnNSum_f(),
            NGLostBarnNSum.NGLostBarnNSum(), decimal=7)



from numba.pycc import CC
from numpy import zeros

cc = CC('AdjUrbanQTotal_inner_compiled')


@cc.export('AdjUrbanQTotal_inner',
           '(int64, int64[:,::1], float64[:,:,::1], float64, float64, float64[:,:,::1], float64[:,:,::1], float64, float64)')
def AdjUrbanQTotal_inner(NYrs, DaysMonth, Temp, Qretention, PctAreaInfil, water, urban_q_total, urb_area_total,
                         area_total):
    result = zeros((NYrs, 12, 31))
    adj_urban_q_total = 0
    for Y in range(NYrs):
        for i in range(12):

            for j in range(DaysMonth[Y][i]):
                if Temp[Y][i][j] > 0 and water[Y][i][j] > 0.01:
                    if water[Y][i][j] < 0.05:
                        # z.adj_urban_q_total = get_value_for_yesterday(z.adj_urban_q_total_1,0,Y,i,j,z.NYrs,z.DaysMonth)
                        # pass
                        adj_urban_q_total *= urb_area_total / area_total
                    else:
                        adj_urban_q_total = urban_q_total[Y][i][j]
                        if Qretention > 0 and urban_q_total[Y][i][j] > 0:
                            if urban_q_total[Y][i][j] <= Qretention * PctAreaInfil:
                                adj_urban_q_total = 0
                            else:
                                adj_urban_q_total = urban_q_total[Y][i][j] - Qretention * PctAreaInfil
                    # if urb_area_total > 0:
                    #     adj_urban_q_total = adj_urban_q_total * urb_area_total / area_total
                    # else:
                    #     adj_urban_q_total = 0
                else:
                    pass
                result[Y][i][j] = adj_urban_q_total
    return result



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns.Erosion import Erosiv


class TestErosiv(VariableUnitTest):

    def test_elementwise_Erosiv(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            np.load(self.basepath + "/Erosiv.npy"),
            Erosiv.Erosiv(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.Acoef), decimal=7)

    def test_Erosiv(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            Erosiv.Erosiv_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.Acoef),
            Erosiv.Erosiv(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.Acoef), decimal=20)



from gwlfe.Memoization import memoize


@memoize
def FilterEff(FilterWidth):
    if FilterWidth <= 30:
        result = FilterWidth / 30
    else:
        result = 1
    return result


# Both have same running time
@memoize
def FilterEff_f(FilterWidth):
    result = 1
    if FilterWidth <= 30:
        result = FilterWidth / 30
    return result



from numpy import maximum
from numpy import zeros

from .InitNgN import InitNgN
from .InitNgN import InitNgN_f


def NGAccManAppN(GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGPctManApp):
    result = zeros((12,))
    init_ng_n = InitNgN(GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN)
    for i in range(12):
        # For Non-Grazing
        result[i] += (init_ng_n / 12) - (NGPctManApp[i] * init_ng_n)
        if result[i] < 0:
            result[i] = 0
    return result


def NGAccManAppN_f(GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGPctManApp):
    init_ng_n = InitNgN_f(GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN)
    return maximum((init_ng_n / 12) - (NGPctManApp * init_ng_n), 0)



from numpy import sum
from numpy import zeros

from gwlfe.Memoization import memoize
from gwlfe.Output.Loading.SedYield_1 import SedYield_1_f


@memoize
def AvSedYield(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
               Grow_0, CNP_0, Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
               RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow,
               StreamWithdrawal, GroundWithdrawal, NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0,
               AvKF, AvSlope, SedAAdjust, StreamLength, n42b, n46c, n85d, AgLength, n42, n45, n85, UrbBankStab,
               Acoef, KF, LS, C, P, SedDelivRatio_0):
    result = zeros(12)
    sedyeild = SedYield_1_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
                            Grow_0, CNP_0, Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap,
                            SatStor_0,
                            RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity,
                            PointFlow,
                            StreamWithdrawal, GroundWithdrawal, NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0,
                            AvKF, AvSlope, SedAAdjust, StreamLength, n42b, n46c, n85d, AgLength, n42, n45, n85,
                            UrbBankStab,
                            Acoef, KF, LS, C, P, SedDelivRatio_0)
    for Y in range(NYrs):
        for i in range(12):
            result[i] += sedyeild[Y][i] / NYrs
    return result


@memoize
def AvSedYield_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
                 Grow_0, CNP_0, Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                 RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow,
                 StreamWithdrawal, GroundWithdrawal, NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0,
                 AvKF, AvSlope, SedAAdjust, StreamLength, n42b, n46c, n85d, AgLength, n42, n45, n85, UrbBankStab,
                 Acoef, KF, LS, C, P, SedDelivRatio_0):
    return sum(
        SedYield_1_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
                     Grow_0, CNP_0, Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                     RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow,
                     StreamWithdrawal, GroundWithdrawal, NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0,
                     AvKF, AvSlope, SedAAdjust, StreamLength, n42b, n46c, n85d, AgLength, n42, n45, n85, UrbBankStab,
                     Acoef, KF, LS, C, P, SedDelivRatio_0),
        axis=0) / NYrs



# -*- coding: utf-8 -*-
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

"""
Imported from LoadReductions.bas
"""

import logging
from .BMPs.Stream.FilterEff import FilterEff

log = logging.getLogger(__name__)


def AdjustScnLoads(z):
    # Check for zero values
    if z.n23 == 0:
        z.n23 = 0.0000001
    if z.n24 == 0:
        z.n24 = 0.0000001
    if z.n42 == 0:
        z.n42 = 0.0000001

    # Estimate sediment reductions for row crops based on ag BMPs
    SROWBMP1 = (z.n25 / 100) * z.n1 * z.n79
    SROWBMP2 = (z.n26 / 100) * z.n1 * z.n81
    SROWBMP3 = (z.n27 / 100) * z.n1 * z.n82
    SROWBMP4 = (z.n27b / 100) * z.n1 * z.n82b
    SROWBMP5 = (z.n28 / 100) * z.n1 * z.n83
    SROWBMP8 = (z.n29 / 100) * z.n1 * z.n84
    SROWRED = z.n1 - (SROWBMP1 + SROWBMP2 + SROWBMP3 + SROWBMP4 + SROWBMP5 + SROWBMP8)
    if (z.SMCheck == "Both" or z.SMCheck == "Upland"):
        if z.n42 > 0:
            SROWSM1 = (z.n43 / z.n42) * SROWRED * z.n80
        if z.n42 > 0:
            SROWSM2 = (z.n46c / z.n42) * SROWRED * z.n85d
        SROWBUF = SROWSM1 + SROWSM2
    else:
        if (z.n42 > 0):
            SROWBUF = (z.n43 / z.n42) * SROWRED * z.n80

    n1Start = z.n1
    z.n1 = SROWRED - SROWBUF
    if (z.n1 < (n1Start * 0.05)):
        z.n1 = n1Start * 0.05

    # Calculate total nitrogen reduction for row crops based on ag BMPs
    NROWBMP6 = (z.n28b / 100) * z.n5 * z.n70
    NROWNM = z.n5 - NROWBMP6
    NROWBMP1 = (z.n25 / 100) * NROWNM * z.n63
    NROWBMP2 = (z.n26 / 100) * NROWNM * z.n65
    NROWBMP3 = (z.n27 / 100) * NROWNM * z.n66
    NROWBMP4 = (z.n27b / 100) * NROWNM * z.n66b
    NROWBMP5 = (z.n28 / 100) * NROWNM * z.n67
    NROWBMP8 = (z.n29 / 100) * NROWNM * z.n68
    NROWRED = NROWNM - (NROWBMP1 + NROWBMP2 + NROWBMP3 + NROWBMP4 + NROWBMP5 + NROWBMP8)
    if (z.SMCheck == "Both" or z.SMCheck == "Upland"):
        if (z.n42 > 0):
            NROWSM1 = (z.n43 / z.n42) * NROWRED * z.n64
        if (z.n42 > 0):
            NROWSM2 = (z.n46c / z.n42) * NROWRED * z.n69c
        NROWBUF = NROWSM1 + NROWSM2
    else:
        if (z.n42 > 0):
            NROWBUF = (z.n43 / z.n42) * NROWRED * z.n64

    n5Start = z.n5
    z.n5 = NROWRED - NROWBUF
    if (z.n5 < (n5Start * 0.05)):
        z.n5 = n5Start * 0.05

    # Calculate dissolved nitrogen reduction for row crops based on ag BMPs
    NROWBMP6 = (z.n28b / 100) * z.n5dn * z.n70
    NROWNM = z.n5dn - NROWBMP6
    NROWBMP1 = (z.n25 / 100) * NROWNM * z.n63
    NROWBMP2 = (z.n26 / 100) * NROWNM * z.n65
    NROWBMP3 = (z.n27 / 100) * NROWNM * z.n66
    NROWBMP4 = (z.n27b / 100) * NROWNM * z.n66b
    NROWBMP5 = (z.n28 / 100) * NROWNM * z.n67
    NROWBMP8 = (z.n29 / 100) * NROWNM * z.n68
    NROWRED = NROWNM - (NROWBMP1 + NROWBMP2 + NROWBMP3 + NROWBMP4 + NROWBMP5 + NROWBMP8)
    if (z.SMCheck == "Both" or z.SMCheck == "Upland"):
        if (z.n42 > 0):
            NROWSM1 = (z.n43 / z.n42) * NROWRED * z.n64
        if (z.n42 > 0):
            NROWSM2 = (z.n46c / z.n42) * NROWRED * z.n69c
        NROWBUF = NROWSM1 + NROWSM2
    else:
        if (z.n42 > 0):
            NROWBUF = (z.n43 / z.n42) * NROWRED * z.n64

    n5dnStart = z.n5dn
    z.n5dn = NROWRED - NROWBUF
    if (z.n5dn < (n5dnStart * 0.05)):
        z.n5dn = n5dnStart * 0.05

    # Calculate total phosphorus reduction for row crops based on ag BMPs
    PROWBMP6 = (z.n28b / 100) * z.n12 * z.n78
    PROWNM = z.n12 - PROWBMP6
    PROWBMP1 = (z.n25 / 100) * PROWNM * z.n71
    PROWBMP2 = (z.n26 / 100) * PROWNM * z.n73
    PROWBMP3 = (z.n27 / 100) * PROWNM * z.n74
    PROWBMP4 = (z.n27b / 100) * PROWNM * z.n74b
    PROWBMP5 = (z.n28 / 100) * PROWNM * z.n75

    PROWBMP8 = (z.n29 / 100) * PROWNM * z.n76
    PROWRED = PROWNM - (PROWBMP1 + PROWBMP2 + PROWBMP3 + PROWBMP4 + PROWBMP5 + PROWBMP8)
    if (z.SMCheck == "Both" or z.SMCheck == "Upland"):
        if (z.n42 > 0):
            PROWSM1 = (z.n43 / z.n42) * PROWRED * z.n72
        if (z.n42 > 0):
            PROWSM2 = (z.n46c / z.n42) * PROWRED * z.n77c
        PROWBUF = PROWSM1 + PROWSM2
    else:
        if (z.n42 > 0):
            PROWBUF = (z.n43 / z.n42) * PROWRED * z.n72

    z.n12Start = z.n12
    z.n12 = PROWRED - PROWBUF
    if (z.n12 < (z.n12Start * 0.05)):
        z.n12 = z.n12Start * 0.05

    # Calculate dissolved phosphorus reduction for row crops based on ag BMPs
    PROWBMP6 = (z.n28b / 100) * z.n12dp * z.n78
    PROWNM = z.n12dp - PROWBMP6
    PROWBMP1 = (z.n25 / 100) * PROWNM * z.n71
    PROWBMP2 = (z.n26 / 100) * PROWNM * z.n73
    PROWBMP3 = (z.n27 / 100) * PROWNM * z.n74
    PROWBMP4 = (z.n27b / 100) * PROWNM * z.n74b
    PROWBMP5 = (z.n28 / 100) * PROWNM * z.n75

    PROWBMP8 = (z.n29 / 100) * PROWNM * z.n76
    PROWRED = PROWNM - (PROWBMP1 + PROWBMP2 + PROWBMP3 + PROWBMP4 + PROWBMP5 + PROWBMP8)
    if (z.SMCheck == "Both" or z.SMCheck == "Upland"):
        if (z.n42 > 0):
            PROWSM1 = (z.n43 / z.n42) * PROWRED * z.n72
        if (z.n42 > 0):
            PROWSM2 = (z.n46c / z.n42) * PROWRED * z.n77c
        PROWBUF = PROWSM1 + PROWSM2
    else:
        if (z.n42 > 0):
            PROWBUF = (z.n43 / z.n42) * PROWRED * z.n72

    n12dpStart = z.n12dp
    z.n12dp = PROWRED - PROWBUF
    if (z.n12dp < (n12dpStart * 0.05)):
        z.n12dp = n12dpStart * 0.05

    # Calculate sed reduction for hay/pasture based on ag BMPs
    SHAYBMP4 = (z.n33c / 100) * z.n2 * z.n82b
    SHAYBMP5 = (z.n35 / 100) * z.n2 * z.n83
    SHAYBMP7 = (z.n36 / 100) * z.n2 * z.n84b
    SHAYBMP8 = (z.n37 / 100) * z.n2 * z.n84

    n2Start = z.n2
    z.n2 = z.n2 - (SHAYBMP4 + SHAYBMP5 + SHAYBMP7 + SHAYBMP8)
    if (z.n2 < (n2Start * 0.05)):
        z.n2 = n2Start * 0.05

    # Calculate total nitrogen reduction for hay/pasture based on different percent usage of BMPs
    NHAYBMP6 = (z.n35b / 100) * z.n6 * z.n70
    NHAYNM = z.n6 - NHAYBMP6
    NHAYBMP4 = (z.n33c / 100) * NHAYNM * z.n66b
    NHAYBMP5 = (z.n35 / 100) * NHAYNM * z.n67
    NHAYBMP7 = (z.n36 / 100) * NHAYNM * z.n68b
    NHAYBMP8 = (z.n37 / 100) * NHAYNM * z.n68

    n6Start = z.n6
    z.n6 = NHAYNM - (NHAYBMP4 + NHAYBMP5 + NHAYBMP7 + NHAYBMP8)
    if (z.n6 < (n6Start * 0.05)):
        z.n6 = n6Start * 0.05

    # Calculate dissolved nitrogen reduction for hay/pasture
    NHAYBMP6 = (z.n35b / 100) * z.n6dn * z.n70
    NHAYNM = z.n6dn - NHAYBMP6
    NHAYBMP4 = (z.n33c / 100) * NHAYNM * z.n66b
    NHAYBMP5 = (z.n35 / 100) * NHAYNM * z.n67
    NHAYBMP7 = (z.n36 / 100) * NHAYNM * z.n68b
    NHAYBMP8 = (z.n37 / 100) * NHAYNM * z.n68

    n6dnStart = z.n6dn
    z.n6dn = NHAYNM - (NHAYBMP4 + NHAYBMP5 + NHAYBMP7 + NHAYBMP8)
    if (z.n6dn < (n6dnStart * 0.05)):
        z.n6dn = n6dnStart * 0.05

    # Calculate total phosphorus reduction for hay/pasture based on different percent usage of BMPs
    PHAYBMP6 = (z.n35b / 100) * z.n13 * z.n78
    PHAYNM = z.n13 - PHAYBMP6
    PHAYBMP4 = (z.n33c / 100) * PHAYNM * z.n74b
    PHAYBMP5 = (z.n35 / 100) * PHAYNM * z.n75
    PHAYBMP7 = (z.n36 / 100) * PHAYNM * z.n76b
    PHAYBMP8 = (z.n37 / 100) * PHAYNM * z.n76

    n13Start = z.n13
    z.n13 = PHAYNM - (PHAYBMP4 + PHAYBMP5 + PHAYBMP7 + PHAYBMP8)
    if (z.n13 < (n13Start * 0.05)):
        z.n13 = n13Start * 0.05

    # Calculate dissolved phosphorus reduction for hay/pasture
    PHAYBMP6 = (z.n35b / 100) * z.n13dp * z.n78
    PHAYNM = z.n13dp - PHAYBMP6
    PHAYBMP4 = (z.n33c / 100) * PHAYNM * z.n74b
    PHAYBMP5 = (z.n35 / 100) * PHAYNM * z.n75
    PHAYBMP7 = (z.n36 / 100) * PHAYNM * z.n76b
    PHAYBMP8 = (z.n37 / 100) * PHAYNM * z.n76

    n13dpStart = z.n13dp
    z.n13dp = PHAYNM - (PHAYBMP4 + PHAYBMP5 + PHAYBMP7 + PHAYBMP8)
    if (z.n13dp < (n13dpStart * 0.05)):
        z.n13dp = n13dpStart * 0.05

    # Calculate phosphorus reduction for animal activities based on different percent of BMPs
    PAWMSL = (z.n41b / 100) * z.n85i * z.GRLBP
    PAWMSP = (z.n41d / 100) * z.n85k * z.NGLBP
    PRUNCON = (z.n41f / 100) * z.n85m * (z.GRLBP + z.NGLBP)
    PHYTASEP = (z.n41h / 100) * z.n85n * (z.NGLManP + z.NGLBP)
    if z.n42 > 0:
        PFENCING = (z.n45 / z.n42) * z.n77 * z.GRSP
        PAGBUFFER = (z.n43 / z.n42) * z.n72 * (z.n14b - (z.NGLBP + z.GRLBP + z.GRSP))

    z.n14b = z.n14b - (PAWMSL + PAWMSP + PRUNCON + PHYTASEP + PFENCING + PAGBUFFER)

    # Calculate Urban Load Reductions

    # High Urban S load
    # Urban Sediment Load Reduction from Wetlands and Streambank Stabilization
    # . . . High-density areas
    SURBWETH = z.n25b * z.n2b * z.n85b
    z.n2b = z.n2b - SURBWETH
    if z.n2b < 0:
        z.n2b = 0

    # . . . Low-density areas
    SURBWETL = z.n25b * z.n2c * z.n85b
    z.n2c = z.n2c - SURBWETL
    if z.n2c < 0:
        z.n2c = 0

    # Urban Nitrogen Load Reduction from Wetlands and Streambank Stabilization
    # . . . High-density areas
    NURBWETH = z.n25b * z.n6b * z.n69b
    z.n6b = z.n6b - NURBWETH
    if z.n6b < 0:
        z.n6b = 0

    # Urban Dissolved Nitrogen Reduction
    NURBWETH = z.n25b * z.n6bdn * z.n69b
    z.n6bdn = z.n6bdn - NURBWETH
    if z.n6bdn < 0:
        z.n6bdn = 0

    # . . . Low-density areas
    NURBWETL = z.n25b * z.n6c * z.n69b
    z.n6c = z.n6c - NURBWETL
    if z.n6c < 0:
        z.n6c = 0

    # Urban Dissolved Nitrogen Reduction
    NURBWETL = z.n25b * z.n6cdn * z.n69b
    z.n6cdn = z.n6cdn - NURBWETL
    if z.n6cdn < 0:
        z.n6cdn = 0

    # Urban Phosphorus Load Reduction from Wetlands and Streambank Stabilization
    # . . . High-density areas
    PURBWETH = z.n25b * z.n13b * z.n77b
    z.n13b = z.n13b - PURBWETH
    if z.n13b < 0:
        z.n13b = 0

    # Urban Dissolved Phosphorus Reduction
    PURBWETH = z.n25b * z.n13bdp * z.n77b
    z.n13bdp = z.n13bdp - PURBWETH
    if z.n13bdp < 0:
        z.n13bdp = 0

    # . . . Low-density areas
    PURBWETL = z.n25b * z.n13c * z.n77b
    z.n13c = z.n13c - PURBWETL
    if z.n13c < 0:
        z.n13c = 0

    # Urban Dissolved Phosphorus Reduction
    PURBWETL = z.n25b * z.n13cdp * z.n77b
    z.n13cdp = z.n13cdp - PURBWETL
    if z.n13cdp < 0:
        z.n13cdp = 0

    # Farm animal FC load reductions
    FCAWMSL = (z.n41b / 100) * z.n85q * z.GRLBFC
    FCAWMSP = (z.n41d / 100) * z.n85r * z.NGLBFC
    FCRUNCON = (z.n41f / 100) * z.n85s * (z.NGLBFC + z.GRLBFC)
    FCFENCING = (z.n45 / z.n42) * z.n85p * z.GRSFC
    FCAGBUFFER = (z.n43 / z.n42) * z.n85o * (z.n139 - (z.NGLBFC + z.GRLBFC + z.GRSFC))
    if FCAGBUFFER < 0:
        FCAGBUFFER = 0

    # For Animal FC
    z.n145 = z.n139 - (FCAWMSL + FCAWMSP + FCRUNCON + FCFENCING + FCAGBUFFER)
    if z.n145 < 0:
        z.n145 = 0

    # For urban FC
    FCURBBIO = z.n142 * z.RetentEff * z.n85u
    FCURBWET = z.n142 * z.n25b * z.n85t
    FCURBBUF = z.n142 * FilterEff(z.FilterWidth) * z.PctStrmBuf * z.n85o
    z.n148 = z.n142 - (FCURBBIO + FCURBWET + FCURBBUF)
    if z.n148 < 0:
        z.n148 = 0

    # Calculations for Unpaved Roads N, P and Sed Load Reduction
    if z.n42c > 0:
        SEDUNPAVED = (z.n46o / z.n42c) * z.n2d * z.n85g * 1.4882
        z.n2d = z.n2d - SEDUNPAVED
        if z.n2d < 0:
            z.n2d = 0

        NUNPAVED = (z.n46o / z.n42c) * z.n6d * z.n85e * 1.4882
        z.n6d = z.n6d - NUNPAVED
        if z.n6d < 0:
            z.n6d = 0

        NUNPAVED = (z.n46o / z.n42c) * z.n6ddn * z.n85e * 1.4882
        z.n6ddn = z.n6ddn - NUNPAVED
        if z.n6ddn < 0:
            z.n6ddn = 0

        PUNPAVED = (z.n46o / z.n42c) * z.n13d * z.n85f * 1.4882
        z.n13d = z.n13d - PUNPAVED
        if z.n13d < 0:
            z.n13d = 0

        PUNPAVED = (z.n46o / z.n42c) * z.n13ddp * z.n85f * 1.4882
        z.n13ddp = z.n13ddp - PUNPAVED
        if z.n13d < 0:
            z.n13ddp = 0



from numpy import maximum
from numpy import minimum
from numpy import ndarray
from numpy import reshape
from numpy import tile
from numpy import zeros

from gwlfe.AFOS.GrazingAnimals.Loads.GRAppManN import GRAppManN
from gwlfe.AFOS.GrazingAnimals.Loads.GRAppManN import GRAppManN_f
from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.LossFactAdj import LossFactAdj
from gwlfe.MultiUse_Fxns.LossFactAdj import LossFactAdj_f


def GRLostManN(NYrs, GRPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRAppNRate, Prec, DaysMonth,
               GRPctSoilIncRate):
    result = zeros((NYrs, 12))
    loss_fact_adj = LossFactAdj(NYrs, Prec, DaysMonth)
    gr_app_man_n = GRAppManN(GRPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN)
    for Y in range(NYrs):
        for i in range(12):
            result[Y][i] = (gr_app_man_n[i] * GRAppNRate[i] * loss_fact_adj[Y][i] * (1 - GRPctSoilIncRate[i]))
            if result[Y][i] > gr_app_man_n[i]:
                result[Y][i] = gr_app_man_n[i]
            if result[Y][i] < 0:
                result[Y][i] = 0
    return result


@memoize
def GRLostManN_f(NYrs, GRPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRAppNRate, Prec, DaysMonth,
                 GRPctSoilIncRate):
    lossFactAdj = LossFactAdj_f(Prec, DaysMonth)
    gr_app_man_n = GRAppManN_f(GRPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN)
    result = (tile(gr_app_man_n, NYrs) * tile(GRAppNRate, NYrs) * ndarray.flatten(lossFactAdj) * tile(
        (1 - GRPctSoilIncRate), NYrs))
    result = minimum(result, tile(gr_app_man_n, NYrs))
    result = maximum(result, 0)
    return reshape(result, (NYrs, 12))



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns.Discharge import StreamFlow_1


class TestStreamFlow_1(VariableUnitTest):

    def test_StreamFlow_1(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            StreamFlow_1.StreamFlow_1_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area,
                                        z.CNI_0, z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper,
                                        z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV, z.PcntET, z.DayHrs, z.MaxWaterCap,
                                        z.SatStor_0, z.RecessionCoef, z.SeepCoef),
            StreamFlow_1.StreamFlow_1(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area,
                                      z.CNI_0, z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper,
                                      z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV, z.PcntET, z.DayHrs, z.MaxWaterCap,
                                      z.SatStor_0, z.RecessionCoef, z.SeepCoef), decimal=7)



from numpy import zeros

from gwlfe.Input.WaterBudget.Percolation import Percolation
from gwlfe.Input.WaterBudget.Percolation import Percolation_f

try:
    from .DeepSeep_inner_compiled import DeepSeep_inner
except ImportError:
    print("Unable to import compiled DeepSeep_inner, using slower version")
    from gwlfe.Input.WaterBudget.DeepSeep_inner import DeepSeep_inner


def SatStorCarryOver(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                     ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef, SeepCoef):
    result = zeros((NYrs, 12, 31))
    deepseep = zeros((NYrs, 12, 31))
    satstor = zeros((NYrs, 12, 31))
    percolation = Percolation(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0,
                              CNP_0,
                              Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap)
    satstor_carryover = SatStor_0
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                satstor[Y][i][j] = satstor_carryover
                result[Y][i][j] = RecessionCoef * satstor[Y][i][j]
                deepseep[Y][i][j] = SeepCoef * satstor[Y][i][j]
                satstor[Y][i][j] = satstor[Y][i][j] + percolation[Y][i][j] - result[Y][i][j] - deepseep[Y][i][j]
                if satstor[Y][i][j] < 0:
                    satstor[Y][i][j] = 0
                satstor_carryover = satstor[Y][i][j]
    return satstor_carryover


def SatStorCarryOver_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0,
                       Imper,
                       ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef,
                       SeepCoef):
    percolation = Percolation_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0,
                                CNP_0,
                                Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap)
    return DeepSeep_inner(NYrs, SatStor_0, DaysMonth, RecessionCoef, SeepCoef, percolation)[3]



from gwlfe.AFOS.nonGrazingAnimals.Losses.NGLostBarnNSum import NGLostBarnNSum


def NAWMSP_2(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGBarnNRate,
             Prec, DaysMonth, AWMSNgPct, NgAWMSCoeffN, RunContPct, RunConCoeffN, n41d, n85j):
    ng_lost_barn_n_sum = NGLostBarnNSum(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN,
                                        NGBarnNRate, Prec,
                                        DaysMonth, AWMSNgPct, NgAWMSCoeffN, RunContPct, RunConCoeffN)
    result = (n41d / 100) * n85j * ng_lost_barn_n_sum
    return result


# def NAWMSP_2_f(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGBarnNRate,
#                Prec, DaysMonth, AWMSNgPct, NgAWMSCoeffN, RunContPct, RunConCoeffN, n41d, n85j):
#     return (n41d / 100) * n85j * AvNGLostBarnNSum_f(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt,
#                                                     AnimalDailyN, NGBarnNRate,
#                                                     Prec, DaysMonth, AWMSNgPct, NgAWMSCoeffN, RunContPct, RunConCoeffN)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns.Runoff import QrunI


class TestQRunI(VariableUnitTest):

    def test_QRunI(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            QrunI.QrunI_f(z.NYrs, z.DaysMonth, z.NRur, z.NUrb, z.Temp, z.InitSnow_0, z.Prec, z.CNI_0, z.AntMoist_0,
                          z.Grow_0),
            QrunI.QrunI(z.NYrs, z.DaysMonth, z.NRur, z.NUrb, z.Temp, z.InitSnow_0, z.Prec, z.CNI_0, z.AntMoist_0,
                        z.Grow_0), decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.AFOS.GrazingAnimals.Loads import GRAccManAppN


class TestGRAccManAppN(VariableUnitTest):

    def test_GRAccManAppN(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            GRAccManAppN.GRAccManAppN(z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN, z.GRPctManApp,
                                      z.PctGrazing)[None, :],
            GRAccManAppN.GRAccManAppN_f(z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN, z.GRPctManApp,
                                        z.PctGrazing), decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.Output.Loading import StreamBankEros_1


class TestStreamBankEros_1(VariableUnitTest):

    def test_StreamBankEros_1(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            StreamBankEros_1.StreamBankEros_1_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb,
                                                z.Area, z.CNI_0, z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper,
                                                z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV, z.PcntET, z.DayHrs,
                                                z.MaxWaterCap, z.SatStor_0, z.RecessionCoef, z.SeepCoef,
                                                z.Qretention, z.PctAreaInfil, z.n25b, z.Landuse, z.TileDrainDensity,
                                                z.PointFlow, z.StreamWithdrawal,
                                                z.GroundWithdrawal, z.NumAnimals, z.AvgAnimalWt, z.StreamFlowVolAdj,
                                                z.SedAFactor_0, z.AvKF, z.AvSlope,
                                                z.SedAAdjust, z.StreamLength, z.n42b, z.n46c, z.n85d, z.AgLength, z.n42,
                                                z.n45, z.n85, z.UrbBankStab),
            StreamBankEros_1.StreamBankEros_1(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area,
                                              z.CNI_0, z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper,
                                              z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV, z.PcntET, z.DayHrs,
                                              z.MaxWaterCap, z.SatStor_0, z.RecessionCoef, z.SeepCoef,
                                              z.Qretention, z.PctAreaInfil, z.n25b, z.Landuse, z.TileDrainDensity,
                                              z.PointFlow, z.StreamWithdrawal,
                                              z.GroundWithdrawal, z.NumAnimals, z.AvgAnimalWt, z.StreamFlowVolAdj,
                                              z.SedAFactor_0, z.AvKF, z.AvSlope,
                                              z.SedAAdjust, z.StreamLength, z.n42b, z.n46c, z.n85d, z.AgLength, z.n42,
                                              z.n45, z.n85, z.UrbBankStab), decimal=7)



from numpy import sum as npsum
from numpy import zeros

from gwlfe.Output.Loading.StreamBankN_1 import StreamBankN_1
from gwlfe.Output.Loading.StreamBankN_1 import StreamBankN_1_f


def AvStreamBankNSum(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                     ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef, SeepCoef,
                     Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow, StreamWithdrawal,
                     GroundWithdrawal, NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF, AvSlope,
                     SedAAdjust, StreamLength, n42b, n46c, n85d, AgLength, n42, n54, n85, UrbBankStab, SedNitr,
                     BankNFrac, n69c, n45, n69):
    AvStreamBankN = zeros(12)
    for Y in range(NYrs):
        for i in range(12):
            AvStreamBankN[i] += \
                StreamBankN_1(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area,
                              CNI_0, AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, CN,
                              UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                              RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse,
                              TileDrainDensity, PointFlow, StreamWithdrawal, GroundWithdrawal,
                              NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF,
                              AvSlope, SedAAdjust, StreamLength, n42b, AgLength,
                              UrbBankStab, SedNitr, BankNFrac, n69c, n45, n69, n46c, n42)[Y][i] / NYrs
    return sum(AvStreamBankN)


def AvStreamBankNSum_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0,
                       Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef,
                       SeepCoef, Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow, StreamWithdrawal,
                       GroundWithdrawal, NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF, AvSlope,
                       SedAAdjust, StreamLength, n42b, n46c, n85d, AgLength, n42, n54, n85, UrbBankStab, SedNitr,
                       BankNFrac, n69c, n45, n69):
    return npsum(StreamBankN_1_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0,
                                 CNP_0, Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                                 RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity,
                                 PointFlow, StreamWithdrawal, GroundWithdrawal, NumAnimals, AvgAnimalWt,
                                 StreamFlowVolAdj,
                                 SedAFactor_0, AvKF, AvSlope, SedAAdjust, StreamLength, n42b, AgLength,
                                 UrbBankStab, SedNitr, BankNFrac, n69c, n45, n69, n46c, n42)) / NYrs



def TotPAEU(NumAnimals, AvgAnimalWt):
    result = 0
    aeu1 = ((NumAnimals[2] / 2) * (AvgAnimalWt[2]) / 1000) + ((NumAnimals[3] / 2) * (AvgAnimalWt[3]) / 1000)
    aeu2 = (NumAnimals[7] * AvgAnimalWt[7]) / 1000
    result += aeu1 + aeu2
    return result


def TotPAEU_f(NumAnimals, AvgAnimalWt):
    return (NumAnimals[2] / 2 * AvgAnimalWt[2] + NumAnimals[3] / 2 * AvgAnimalWt[3] + NumAnimals[7] * AvgAnimalWt[
        7]) / 1000



from gwlfe.Input.WaterBudget.FlowDays import FlowDays
from gwlfe.Memoization import memoize


@memoize
def AttenN(AttenFlowDist, AttenFlowVel, AttenLossRateN):
    return FlowDays(AttenFlowDist, AttenFlowVel) * AttenLossRateN

# def AttenN_f():
#     pass



from numpy import sum
from numpy import zeros

from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Erosion.Erosion_1 import Erosion_1
from gwlfe.MultiUse_Fxns.Erosion.Erosion_1 import Erosion_1_f


def AvErosion(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
              Grow_0, CNP_0, Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
              RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow,
              StreamWithdrawal, GroundWithdrawal, NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0,
              AvKF, AvSlope, SedAAdjust, StreamLength, n42b, n46c, n85d, AgLength, n42, n45, n85, UrbBankStab,
              SedDelivRatio_0, Acoef, KF, LS, C, P):
    result = zeros(12)
    erosion = Erosion_1(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
                        Grow_0, CNP_0, Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                        RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow,
                        StreamWithdrawal, GroundWithdrawal, NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0,
                        AvKF, AvSlope, SedAAdjust, StreamLength, n42b, n46c, n85d, AgLength, n42, n45, n85, UrbBankStab,
                        SedDelivRatio_0, Acoef, KF, LS, C, P)
    for Y in range(NYrs):
        for i in range(12):
            result[i] += erosion[Y][i] / NYrs
    return result


@memoize
def AvErosion_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
                Grow_0, CNP_0, Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow,
                StreamWithdrawal, GroundWithdrawal, NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0,
                AvKF, AvSlope, SedAAdjust, StreamLength, n42b, n46c, n85d, AgLength, n42, n45, n85, UrbBankStab,
                SedDelivRatio_0, Acoef, KF, LS, C, P):
    return sum(Erosion_1_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
                           Grow_0, CNP_0, Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap,
                           SatStor_0,
                           RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity,
                           PointFlow,
                           StreamWithdrawal, GroundWithdrawal, NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0,
                           AvKF, AvSlope, SedAAdjust, StreamLength, n42b, n46c, n85d, AgLength, n42, n45, n85,
                           UrbBankStab,
                           SedDelivRatio_0, Acoef, KF, LS, C, P), axis=0) / NYrs



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.AFOS.nonGrazingAnimals.Loads import NGAppManN


class TestNGAppManN(VariableUnitTest):
    def test_NGAppManN(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            NGAppManN.NGAppManN_f(z.NGPctManApp, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN),
            NGAppManN.NGAppManN(z.NGPctManApp, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN),
            decimal=7)



from numpy import average
from numpy import float64
from numpy import sum
from numpy import zeros

from gwlfe.Memoization import memoize


def Precipitation(NYrs, DaysMonth, Prec):  # TODO: change internal "Precipitation" to "result"
    Precipitation = zeros((NYrs, 12))
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                Precipitation[Y][i] = Precipitation[Y][i] + Prec[Y][i][j]
    return Precipitation


@memoize
def Precipitation_f(Prec):
    return sum(Prec, dtype=float64, axis=(2))


@memoize
def AvPrecipitation_f(Precipitation):
    return average(Precipitation, axis=0)



from numpy import sum
from numpy import zeros

from gwlfe.Input.LandUse.LU import LU
from gwlfe.Input.LandUse.NLU import NLU
from gwlfe.Input.WaterBudget.Water import Water
from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Runoff.QrunI import QrunI
from gwlfe.MultiUse_Fxns.Runoff.QrunI import QrunI_f
from gwlfe.MultiUse_Fxns.Runoff.QrunP import QrunP
from gwlfe.MultiUse_Fxns.Runoff.QrunP import QrunP_f


@memoize
def UrbQRunoff(NYrs, DaysMonth, InitSnow_0, Temp, Prec, NRur, NUrb, CNI_0, CNP_0, AntMoist_0, Grow_0, Imper, ISRR,
               ISRA):
    result = zeros((NYrs, 16, 12))
    nlu = NLU(NRur, NUrb)
    water = Water(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    qruni = QrunI(NYrs, DaysMonth, NRur, NUrb, Temp, InitSnow_0, Prec, CNI_0, AntMoist_0, Grow_0)
    qrunp = QrunP(NYrs, DaysMonth, NRur, NUrb, Temp, InitSnow_0, Prec, CNP_0, AntMoist_0, Grow_0)
    lu = LU(NRur, NUrb)
    for Y in range(NYrs):
        for i in range(12):
            for l in range(nlu):
                result[Y, l, i] = 0
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                if Temp[Y][i][j] > 0 and water[Y][i][j] > 0.01:
                    if water[Y][i][j] < 0.05:
                        pass
                    else:
                        for l in range(NRur, nlu):
                            result[Y][l][i] += (qruni[Y][i][j][l] * (
                                    Imper[l] * (1 - ISRR[lu[l]]) * (1 - ISRA[lu[l]]))
                                                + qrunp[Y][i][j][l] * (
                                                        1 - (Imper[l] * (1 - ISRR[lu[l]]) * (
                                                        1 - ISRA[lu[l]]))))

                else:
                    pass
    return result


def UrbQRunoff_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec, NRur, NUrb, CNI_0, CNP_0, AntMoist_0, Grow_0, Imper, ISRR,
                 ISRA):
    qruni = QrunI_f(NYrs, DaysMonth, NRur, NUrb, Temp, InitSnow_0, Prec, CNI_0, AntMoist_0, Grow_0)[:, :, :, NRur:]
    qrunp = QrunP_f(NYrs, DaysMonth, NRur, NUrb, Temp, InitSnow_0, Prec, CNP_0, AntMoist_0, Grow_0)[:, :, :, NRur:]
    temp = (Imper[NRur:] * (1 - ISRR) * (1 - ISRA))
    return sum(qruni * temp + qrunp * (1 - temp), axis=2)



from numpy import zeros

from gwlfe.Input.WaterBudget.ET import DailyET_f
from gwlfe.Input.WaterBudget.Infiltration import Infiltration
from gwlfe.Input.WaterBudget.Infiltration import Infiltration_f
from gwlfe.Memoization import memoize

try:
    from .Percolation_inner_compiled import Percolation_inner
except ImportError:
    print("Unable to import compiled Percolation_inner, using slower version")
    from gwlfe.Input.WaterBudget.Percolation_inner import Percolation_inner


@memoize
def Percolation(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap):
    result = zeros((NYrs, 12, 31))
    percolation = zeros((NYrs, 12, 31))
    infiltration = Infiltration(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0,
                                CNP_0, Imper,
                                ISRR, ISRA, CN)
    unsatstor_carryover = UnsatStor_0
    et = DailyET_f(Temp, KV, PcntET, DayHrs)
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                result[Y][i][j] = unsatstor_carryover
                result[Y][i][j] = result[Y][i][j] + infiltration[Y][i][j]
                if et[Y][i][j] >= result[Y][i][j]:
                    result[Y][i][j] = 0
                else:
                    result[Y][i][j] = result[Y][i][j] - et[Y][i][j]
                if result[Y][i][j] > MaxWaterCap:
                    percolation[Y][i][j] = result[Y][i][j] - MaxWaterCap
                    result[Y][i][j] = MaxWaterCap
                else:
                    pass
                unsatstor_carryover = result[Y][i][j]
    return percolation


def Percolation_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                  ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap):
    infiltration = Infiltration_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0,
                                  CNP_0, Imper, ISRR, ISRA, CN)

    et = DailyET_f(Temp, KV, PcntET, DayHrs)
    return Percolation_inner(NYrs, UnsatStor_0, DaysMonth, MaxWaterCap, infiltration, et)



from gwlfe.AFOS.GrazingAnimals.Losses.GRLBN import GRLBN
from gwlfe.AFOS.GrazingAnimals.Losses.GRLBN import GRLBN_f
from gwlfe.AFOS.nonGrazingAnimals.Losses.NGLostBarnNSum import NGLostBarnNSum
from gwlfe.AFOS.nonGrazingAnimals.Losses.NGLostBarnNSum import NGLostBarnNSum_f


def NRUNCON_2(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp, PctGrazing, GRBarnNRate,
              Prec, DaysMonth, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN, NGPctManApp, NGBarnNRate, AWMSNgPct,
              NgAWMSCoeffN, n41f, n85l):
    grlbn = GRLBN(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp, PctGrazing, GRBarnNRate,
                  Prec, DaysMonth, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN)
    nglbn = NGLostBarnNSum(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGBarnNRate,
                           Prec, DaysMonth, AWMSNgPct, NgAWMSCoeffN, RunContPct, RunConCoeffN)
    result = (n41f / 100) * n85l * (grlbn + nglbn)
    return result


# def NRUNCON_2_f(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp, PctGrazing, GRBarnNRate,
#                 Prec, DaysMonth, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN, NGPctManApp, NGBarnNRate, AWMSNgPct,
#                 NgAWMSCoeffN, n41f, n85l):
#     grlbn = GRLBN_f(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp, PctGrazing, GRBarnNRate,
#                     Prec, DaysMonth, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN)
#     nglbn = NGLostBarnNSum_f(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGBarnNRate,
#                              Prec, DaysMonth, AWMSNgPct, NgAWMSCoeffN, RunContPct, RunConCoeffN)
#     return (n41f / 100) * n85l * (grlbn + nglbn)



# -*- coding: utf-8 -*-
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

"""
Imported from CalcLoads.bas
"""

from numpy import zeros

from .Input.LandUse.AreaTotal import AreaTotal_f
from .Input.WaterBudget.GroundWatLE_1 import GroundWatLE_1_f
from .Input.LandUse.Ag.TileDrain import TileDrain_f
from .Input.LandUse.TotAreaMeters import TotAreaMeters
from .MultiUse_Fxns.Erosion.SedDelivRatio import SedDelivRatio
from .MultiUse_Fxns.Erosion.Erosion_1 import Erosion_1_f
from .Output.Loading.SedYield import SedYield_f
from .Output.Loading.LuLoad import LuLoad_f
from .Output.Loading.LuDisLoad import LuDisLoad_f
from .MultiUse_Fxns.Erosion.LuErosion import LuErosion_f
from .MultiUse_Fxns.Runoff.nRunoff import nRunoff_f
from .MultiUse_Fxns.Runoff.pRunoff import pRunoff_f
from .Output.Loading.DisLoad import DisLoad_f
from .Output.Loading.Load import Load_f


def CalculateLoads(z, Y):
    GroundWatLETotal = zeros(z.WxYrs)
    SedYieldTotal = 0
    ErosionTotal = 0
    DisNitrTotal = 0
    DisPhosTotal = 0
    TotNitrTotal = 0
    TotPhosTotal = 0
    AnimalFCTotal = 0
    WWOrgsTotal = 0
    SSOrgsTotal = 0
    UrbOrgsTotal = 0
    WildOrgsTotal = 0
    TotalOrgsTotal = 0
    CMStreamTotal = 0
    OrgConcTotal = 0

    # ANNUAL WATER BALANCE CALCULATIONS
    for i in range(12):
        # Calculate landuse runoff for rural areas
        GroundWatLETotal += \
            GroundWatLE_1_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                            z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper,
                            z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV, z.PcntET, z.DayHrs, z.MaxWaterCap,
                            z.SatStor_0, z.RecessionCoef, z.SeepCoef,
                            z.Landuse, z.TileDrainDensity)[Y][i]

    # CALCULATE ANNUAL NITROGEN  LOADS FROM NORMAL SEPTIC SYSTEMS
    AnNormNitr = 0
    for i in range(12):
        AnNormNitr += z.MonthNormNitr[i] * z.NumNormalSys[i]

    z.CalendarYr = z.WxYrBeg + (Y - 1)

    # SEDIMENT YIELD AND TILE DRAINAGE
    for i in range(12):
        SedYieldTotal += \
            SedYield_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.Acoef, z.NRur, z.KF, z.LS, z.C, z.P,
                       z.Area, z.NUrb, z.CNI_0, z.AntMoist_0, z.Grow_0, z.ISRR, z.ISRA, z.Qretention, z.PctAreaInfil,
                       z.n25b, z.CN, z.CNP_0, z.Imper, z.SedDelivRatio_0)[Y][i]
        ErosionTotal += \
            Erosion_1_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                        z.AntMoist_0,
                        z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV, z.PcntET, z.DayHrs,
                        z.MaxWaterCap, z.SatStor_0,
                        z.RecessionCoef, z.SeepCoef, z.Qretention, z.PctAreaInfil, z.n25b, z.Landuse,
                        z.TileDrainDensity, z.PointFlow,
                        z.StreamWithdrawal, z.GroundWithdrawal, z.NumAnimals, z.AvgAnimalWt, z.StreamFlowVolAdj,
                        z.SedAFactor_0,
                        z.AvKF, z.AvSlope, z.SedAAdjust, z.StreamLength, z.n42b, z.n46c, z.n85d, z.AgLength, z.n42,
                        z.n45, z.n85, z.UrbBankStab,
                        z.SedDelivRatio_0, z.Acoef, z.KF, z.LS, z.C, z.P)[Y][i]

        # CALCULATION OF THE LANDUSE EROSION AND SEDIMENT YIELDS
        for l in range(z.NRur):
            z.LuSedYield[Y][l] = \
                LuErosion_f(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.NRur, z.Acoef, z.KF, z.LS,
                            z.C, z.P, z.Area)[Y][l] * SedDelivRatio(z.SedDelivRatio_0)
            z.DisNitr[Y][i] += \
                nRunoff_f(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.AntMoist_0, z.NRur, z.NUrb, z.CN,
                          z.Grow_0,
                          z.Area, z.NitrConc, z.ManNitr, z.ManuredAreas, z.FirstManureMonth, z.LastManureMonth,
                          z.FirstManureMonth2, z.LastManureMonth2)[Y][i][l]
            z.DisPhos[Y][i] += \
                pRunoff_f(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.AntMoist_0, z.NRur, z.NUrb, z.CN,
                          z.Grow_0,
                          z.Area, z.PhosConc, z.ManuredAreas, z.FirstManureMonth, z.LastManureMonth, z.ManPhos,
                          z.FirstManureMonth2, z.LastManureMonth2)[Y][i][l]
            z.LuDisNitr[Y][l] += \
                nRunoff_f(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.AntMoist_0, z.NRur, z.NUrb, z.CN,
                          z.Grow_0,
                          z.Area, z.NitrConc, z.ManNitr, z.ManuredAreas, z.FirstManureMonth, z.LastManureMonth,
                          z.FirstManureMonth2, z.LastManureMonth2)[Y][i][l]
            z.LuDisPhos[Y][l] += \
                pRunoff_f(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.AntMoist_0, z.NRur, z.NUrb, z.CN,
                          z.Grow_0,
                          z.Area, z.PhosConc, z.ManuredAreas, z.FirstManureMonth, z.LastManureMonth, z.ManPhos,
                          z.FirstManureMonth2, z.LastManureMonth2)[Y][i][l]

        z.TotNitr[Y][i] = z.DisNitr[Y][i] + 0.001 * z.SedNitr * \
                          SedYield_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.Acoef, z.NRur, z.KF, z.LS,
                                     z.C, z.P,
                                     z.Area, z.NUrb, z.CNI_0, z.AntMoist_0, z.Grow_0, z.ISRR, z.ISRA, z.Qretention,
                                     z.PctAreaInfil,
                                     z.n25b, z.CN, z.CNP_0, z.Imper, z.SedDelivRatio_0)[Y][i]
        z.TotPhos[Y][i] = z.DisPhos[Y][i] + 0.001 * z.SedPhos * \
                          SedYield_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.Acoef, z.NRur, z.KF, z.LS,
                                     z.C, z.P,
                                     z.Area, z.NUrb, z.CNI_0, z.AntMoist_0, z.Grow_0, z.ISRR, z.ISRA, z.Qretention,
                                     z.PctAreaInfil,
                                     z.n25b, z.CN, z.CNP_0, z.Imper, z.SedDelivRatio_0)[Y][i]

        # SUM TILE DRAIN N, P, AND SEDIMENT
        z.TileDrainN[Y][i] += ((((TileDrain_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area,
                                              z.CNI_0,
                                              z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper,
                                              z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV, z.PcntET, z.DayHrs,
                                              z.MaxWaterCap, z.SatStor_0,
                                              z.RecessionCoef, z.SeepCoef, z.Landuse,
                                              z.TileDrainDensity)[Y][i] / 100) * TotAreaMeters(z.NRur, z.NUrb,
                                                                                               z.Area)) * 1000) * z.TileNconc) / 1000000
        z.TileDrainP[Y][i] += ((((TileDrain_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area,
                                              z.CNI_0,
                                              z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper,
                                              z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV, z.PcntET, z.DayHrs,
                                              z.MaxWaterCap, z.SatStor_0,
                                              z.RecessionCoef, z.SeepCoef, z.Landuse,
                                              z.TileDrainDensity)[Y][i] / 100) * TotAreaMeters(z.NRur, z.NUrb,
                                                                                               z.Area)) * 1000) * z.TilePConc) / 1000000
        z.TileDrainSed[Y][i] += ((((TileDrain_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb,
                                                z.Area, z.CNI_0,
                                                z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper,
                                                z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV, z.PcntET, z.DayHrs,
                                                z.MaxWaterCap, z.SatStor_0,
                                                z.RecessionCoef, z.SeepCoef, z.Landuse,
                                                z.TileDrainDensity)[Y][i] / 100) * TotAreaMeters(z.NRur, z.NUrb,

                                                                                                 z.Area)) * 1000) * z.TileSedConc) / 1000000
        z.LuDisNitr[Y, z.NRur:] += \
            LuDisLoad_f(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.Nqual, z.NRur, z.NUrb, z.Area, z.CNI_0,
                        z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.Qretention, z.PctAreaInfil,
                        z.LoadRateImp, z.LoadRatePerv, z.Storm, z.UrbBMPRed, z.DisFract,
                        z.FilterWidth, z.PctStrmBuf)[Y, :, 0] / z.NYrs / 2
        z.LuDisPhos[Y, z.NRur:] += \
            LuDisLoad_f(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.Nqual, z.NRur, z.NUrb, z.Area, z.CNI_0,
                        z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.Qretention, z.PctAreaInfil,
                        z.LoadRateImp, z.LoadRatePerv, z.Storm, z.UrbBMPRed, z.DisFract,
                        z.FilterWidth, z.PctStrmBuf)[Y, :, 1] / z.NYrs / 2
        z.LuSedYield[Y, z.NRur:] += (LuLoad_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area,
                                              z.CNI_0, z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA,
                                              z.Qretention, z.PctAreaInfil, z.Nqual, z.LoadRateImp, z.LoadRatePerv,
                                              z.Storm, z.UrbBMPRed,
                                              z.FilterWidth, z.PctStrmBuf)[Y, :, 2] / z.NYrs) / 1000 / 2

        z.DisNitr[Y][i] += \
        DisLoad_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0, z.AntMoist_0,
                  z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.Qretention, z.PctAreaInfil, z.Nqual, z.LoadRateImp,
                  z.LoadRatePerv, z.Storm, z.UrbBMPRed, z.DisFract, z.FilterWidth, z.PctStrmBuf)[Y][i][0]
        z.DisPhos[Y][i] += \
        DisLoad_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0, z.AntMoist_0,
                  z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.Qretention, z.PctAreaInfil, z.Nqual, z.LoadRateImp,
                  z.LoadRatePerv, z.Storm, z.UrbBMPRed, z.DisFract, z.FilterWidth, z.PctStrmBuf)[Y][i][1]
        z.TotNitr[Y][i] += \
        Load_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0, z.AntMoist_0,
               z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.Qretention, z.PctAreaInfil, z.Nqual, z.LoadRateImp,
               z.LoadRatePerv, z.Storm, z.UrbBMPRed, z.DisFract, z.FilterWidth, z.PctStrmBuf)[Y][i][0]
        z.TotPhos[Y][i] += \
        Load_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0, z.AntMoist_0,
               z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.Qretention, z.PctAreaInfil, z.Nqual, z.LoadRateImp,
               z.LoadRatePerv, z.Storm, z.UrbBMPRed, z.DisFract, z.FilterWidth, z.PctStrmBuf)[Y][i][1]

        # ADD UPLAND N and P LOADS
        z.UplandN[Y][i] = z.TotNitr[Y][i]
        z.UplandP[Y][i] = z.TotPhos[Y][i]

        # ADD GROUNDWATER, POINT SOURCES,
        z.GroundNitr[Y][i] = 0.1 * z.GrNitrConc * \
                             GroundWatLE_1_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area,
                                             z.CNI_0,
                                             z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper,
                                             z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV, z.PcntET, z.DayHrs,
                                             z.MaxWaterCap,
                                             z.SatStor_0, z.RecessionCoef, z.SeepCoef,
                                             z.Landuse, z.TileDrainDensity)[Y][i] * AreaTotal_f(z.Area)
        z.GroundPhos[Y][i] = 0.1 * z.GrPhosConc * \
                             GroundWatLE_1_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area,
                                             z.CNI_0,
                                             z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper,
                                             z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV, z.PcntET, z.DayHrs,
                                             z.MaxWaterCap,
                                             z.SatStor_0, z.RecessionCoef, z.SeepCoef,
                                             z.Landuse, z.TileDrainDensity)[Y][i] * AreaTotal_f(z.Area)
        z.DisNitr[Y][i] += z.GroundNitr[Y][i] + z.PointNitr[i]
        z.DisPhos[Y][i] += z.GroundPhos[Y][i] + z.PointPhos[i]
        z.TotNitr[Y][i] += z.GroundNitr[Y][i] + z.PointNitr[i]
        z.TotPhos[Y][i] += z.GroundPhos[Y][i] + z.PointPhos[i]

        # ADD SEPTIC SYSTEM SOURCES TO MONTHLY DISSOLVED NUTRIENT TOTALS
        if GroundWatLETotal[Y] <= 0:
            GroundWatLETotal[Y] = 0.0001

        z.MonthNormNitr[i] = AnNormNitr * \
                             GroundWatLE_1_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area,
                                             z.CNI_0,
                                             z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper,
                                             z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV, z.PcntET, z.DayHrs,
                                             z.MaxWaterCap,
                                             z.SatStor_0, z.RecessionCoef, z.SeepCoef,
                                             z.Landuse, z.TileDrainDensity)[Y][i] / GroundWatLETotal[Y]

        z.DisSeptNitr = (z.MonthNormNitr[i]
                         + z.MonthPondNitr[i]
                         + z.MonthShortNitr[i] * z.NumShortSys[i]
                         + z.MonthDischargeNitr[i] * z.NumDischargeSys[i])

        z.DisSeptPhos = (z.MonthPondPhos[i]
                         + z.MonthShortPhos[i] * z.NumShortSys[i]
                         + z.MonthDischargePhos[i] * z.NumDischargeSys[i])

        # 0.59 IS ATTENUATION FACTOR FOR SOIL LOSS
        # 0.66 IS ATTENUATION FACTOR FOR SUBSURFACE FLOW LOSS
        z.DisSeptNitr = z.DisSeptNitr / 1000 * 0.59 * 0.66
        z.DisSeptPhos = z.DisSeptPhos / 1000

        z.DisNitr[Y][i] += z.DisSeptNitr
        z.DisPhos[Y][i] += z.DisSeptPhos
        z.TotNitr[Y][i] += z.DisSeptNitr
        z.TotPhos[Y][i] += z.DisSeptPhos
        z.SepticN[Y][i] += z.DisSeptNitr
        z.SepticP[Y][i] += z.DisSeptPhos

        # ANNUAL TOTALS
        DisNitrTotal += z.DisNitr[Y][i]
        DisPhosTotal += z.DisPhos[Y][i]
        TotNitrTotal += z.TotNitr[Y][i]
        TotPhosTotal += z.TotPhos[Y][i]

        # UPDATE ANNUAL SEPTIC SYSTEM LOADS
        z.SepticNitr[Y] += z.DisSeptNitr
        z.SepticPhos[Y] += z.DisSeptPhos

        # Annual pathogen totals
        AnimalFCTotal += z.AnimalFC[Y][i]
        WWOrgsTotal += z.WWOrgs[Y][i]
        SSOrgsTotal += z.SSOrgs[Y][i]
        UrbOrgsTotal += z.UrbOrgs[Y][i]
        WildOrgsTotal += z.WildOrgs[Y][i]
        TotalOrgsTotal += z.TotalOrgs[Y][i]
        CMStreamTotal += z.CMStream[Y][i]
        OrgConcTotal += z.OrgConc[Y][i]



from numpy import sum
from numpy import zeros

from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Discharge.Flow import Flow
from gwlfe.MultiUse_Fxns.Discharge.Flow import Flow_f


@memoize
def StreamFlow_1(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                 ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef, SeepCoef):
    result = zeros((NYrs, 12))
    flow = Flow(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef, SeepCoef)

    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                result[Y][i] = result[Y][i] + flow[Y][i][j]

    return result


@memoize
def StreamFlow_1_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                   ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef, SeepCoef):
    flow = Flow_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                  ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef, SeepCoef)
    return sum(flow, axis=2)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns.Erosion import AvStreamBankEros


class TestAvStreamBankEros(VariableUnitTest):

    def test_AvStreamBankEros(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            AvStreamBankEros.AvStreamBankEros_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb,
                                                z.Area, z.CNI_0, z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper,
                                                z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV, z.PcntET, z.DayHrs,
                                                z.MaxWaterCap, z.SatStor_0, z.RecessionCoef, z.SeepCoef,
                                                z.Qretention, z.PctAreaInfil, z.n25b, z.Landuse, z.TileDrainDensity,
                                                z.PointFlow, z.StreamWithdrawal, z.GroundWithdrawal, z.NumAnimals,
                                                z.AvgAnimalWt, z.StreamFlowVolAdj, z.SedAFactor_0, z.AvKF, z.AvSlope,
                                                z.SedAAdjust, z.StreamLength, z.n42b, z.n46c, z.n85d, z.AgLength, z.n42,
                                                z.n45, z.n85, z.UrbBankStab),
            AvStreamBankEros.AvStreamBankEros(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area,
                                              z.CNI_0, z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN,
                                              z.UnsatStor_0, z.KV, z.PcntET, z.DayHrs, z.MaxWaterCap, z.SatStor_0,
                                              z.RecessionCoef, z.SeepCoef, z.Qretention, z.PctAreaInfil, z.n25b,
                                              z.Landuse, z.TileDrainDensity, z.PointFlow, z.StreamWithdrawal,
                                              z.GroundWithdrawal, z.NumAnimals, z.AvgAnimalWt, z.StreamFlowVolAdj,
                                              z.SedAFactor_0, z.AvKF, z.AvSlope, z.SedAAdjust, z.StreamLength, z.n42b,
                                              z.n46c, z.n85d, z.AgLength, z.n42, z.n45, z.n85, z.UrbBankStab),
            decimal=7)



from numba.pycc import CC
from numpy import zeros

cc = CC('DeepSeep_inner_compiled')


@cc.export('DeepSeep_inner', '(int64, float64, int64[:,::1], float64, float64, float64[:,:,::1])')
def DeepSeep_inner(NYrs, SatStor_0, DaysMonth, RecessionCoef, SeepCoef, percolation):
    deepseep = zeros((NYrs, 12, 31))
    grflow = zeros((NYrs, 12, 31))
    satstor = zeros((NYrs, 12, 31))
    satstor_carryover = SatStor_0
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                satstor[Y][i][j] = satstor_carryover
                grflow[Y][i][j] = max(RecessionCoef * satstor[Y][i][j],1e-20)
                deepseep[Y][i][j] = SeepCoef * satstor[Y][i][j]
                satstor[Y][i][j] = satstor[Y][i][j] + percolation[Y][i][j] - grflow[Y][i][j] - deepseep[Y][i][j]
                if satstor[Y][i][j] < 0:
                    satstor[Y][i][j] = 0
                # if(satstor[Y][i][j] < 1e-300):
                #     satstor_carryover = 0
                # else:
                satstor_carryover = satstor[Y][i][j]
    return deepseep, grflow, satstor, satstor_carryover



from numpy import repeat
from numpy import zeros

from gwlfe.Memoization import memoize


@memoize
def PtSrcFlow(NYrs, PointFlow):
    result = zeros((NYrs, 12))
    for Y in range(NYrs):
        for i in range(12):
            result[Y][i] = result[Y][i] + PointFlow[i]
    return result


def PtSrcFlow_f(NYrs, PointFlow):
    return repeat(PointFlow[:, None], NYrs, axis=1).T


def AvPtSrcFlow(NYrs, PtSrcFlow):
    result = zeros((12,))
    for Y in range(NYrs):
        for i in range(12):
            result[i] += PtSrcFlow[Y][i] / NYrs
    return result


@memoize
def AvPtSrcFlow_f(PointFlow):
    return PointFlow



from numpy import zeros

from gwlfe.Output.Loading.StreamBankN import StreamBankN
from gwlfe.Output.Loading.StreamBankN import StreamBankN_f


def NURBBANK(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area,
             CNI_0, AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, CN,
             UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
             RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse,
             TileDrainDensity, PointFlow, StreamWithdrawal, GroundWithdrawal,
             NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF,
             AvSlope, SedAAdjust, StreamLength, n42b, UrbBankStab, SedNitr, BankNFrac, n69c):
    result = zeros((NYrs, 12))
    streambank_n = StreamBankN(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area,
                               CNI_0, AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, CN,
                               UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                               RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse,
                               TileDrainDensity, PointFlow, StreamWithdrawal, GroundWithdrawal,
                               NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF,
                               AvSlope, SedAAdjust, StreamLength, SedNitr, BankNFrac)
    for Y in range(NYrs):
        for i in range(12):
            if n42b > 0:
                result[Y][i] = (UrbBankStab / n42b) * streambank_n[Y][i] * n69c
    return result


def NURBBANK_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area,
               CNI_0, AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, CN,
               UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
               RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse,
               TileDrainDensity, PointFlow, StreamWithdrawal, GroundWithdrawal,
               NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF,
               AvSlope, SedAAdjust, StreamLength, n42b, UrbBankStab, SedNitr, BankNFrac, n69c):
    if n42b > 0:
        return (UrbBankStab / n42b) * StreamBankN_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area,
                                                    CNI_0, AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, CN,
                                                    UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                                                    RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse,
                                                    TileDrainDensity, PointFlow, StreamWithdrawal, GroundWithdrawal,
                                                    NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF,
                                                    AvSlope, SedAAdjust, StreamLength, SedNitr, BankNFrac) * n69c
    else:
        return zeros((NYrs, 12))



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.Output.Loading import DisSurfLoad


class TestDisSurfLoad(VariableUnitTest):

    def test_DisSurfLoad(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            DisSurfLoad.DisSurfLoad_f(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.Nqual, z.NRur, z.NUrb,
                                      z.Area, z.CNI_0, z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA,
                                      z.Qretention, z.PctAreaInfil, z.LoadRateImp, z.LoadRatePerv,
                                      z.Storm, z.UrbBMPRed, z.DisFract, z.FilterWidth, z.PctStrmBuf),
            DisSurfLoad.DisSurfLoad(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.Nqual, z.NRur, z.NUrb,
                                    z.Area, z.CNI_0, z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA,
                                    z.Qretention, z.PctAreaInfil, z.LoadRateImp, z.LoadRatePerv,
                                    z.Storm, z.UrbBMPRed, z.DisFract, z.FilterWidth, z.PctStrmBuf)[:, :, :, z.NRur:],
            decimal=7)
